@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2012 Douglas Gilbert.
+ * Copyright (c) 1999-2015 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -16,6 +16,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 #include <unistd.h>
 #include "sg_lib.h"
@@ -26,8 +27,13 @@
 #include "config.h"
 #endif
 
+/* Needs to be after config.h */
+#ifdef SG_LIB_LINUX
+#include <errno.h>
+#endif
 
-static char * version_str = "1.57 20120328";
+
+static const char * version_str = "1.70 20150511";
 
 
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
@@ -46,7 +52,7 @@ static char * version_str = "1.57 20120328";
 #define TUR_CMD  0x0
 #define TUR_CMDLEN  6
 
-#define INQUIRY_RESP_INITIAL_LEN 36
+#define SAFE_STD_INQ_RESP_LEN 36 /* other lengths lock up some devices */
 
 
 const char *
@@ -55,11 +61,37 @@ sg_cmds_version()
     return version_str;
 }
 
+#ifdef __GNUC__
+static int pr2ws(const char * fmt, ...)
+        __attribute__ ((format (printf, 1, 2)));
+#else
+static int pr2ws(const char * fmt, ...);
+#endif
+
+
+static int
+pr2ws(const char * fmt, ...)
+{
+    va_list args;
+    int n;
+
+    va_start(args, fmt);
+    n = vfprintf(sg_warnings_strm ? sg_warnings_strm : stderr, fmt, args);
+    va_end(args);
+    return n;
+}
+
 /* Returns file descriptor >= 0 if successful. If error in Unix returns
    negated errno. */
 int
 sg_cmds_open_device(const char * device_name, int read_only, int verbose)
 {
+    /* The following 2 lines are temporary. It is to avoid a NULL pointer
+     * crash when an old utility is used with a newer library built after
+     * the sg_warnings_strm cleanup */
+    if (NULL == sg_warnings_strm)
+        sg_warnings_strm = stderr;
+
     return scsi_pt_open_device(device_name, read_only, verbose);
 }
 
@@ -91,16 +123,22 @@ sg_cmds_process_helper(const char * leadin, int mx_di_len, int resid,
     scat = sg_err_category_sense(sbp, slen);
     switch (scat) {
     case SG_LIB_CAT_NOT_READY:
-    case SG_LIB_CAT_UNIT_ATTENTION:
     case SG_LIB_CAT_INVALID_OP:
     case SG_LIB_CAT_ILLEGAL_REQ:
     case SG_LIB_CAT_ABORTED_COMMAND:
+    case SG_LIB_CAT_COPY_ABORTED:
+    case SG_LIB_CAT_DATA_PROTECT:
+    case SG_LIB_CAT_PROTECTION:
     case SG_LIB_CAT_NO_SENSE:
+    case SG_LIB_CAT_MISCOMPARE:
         n = 0;
         break;
     case SG_LIB_CAT_RECOVERED:
     case SG_LIB_CAT_MEDIUM_HARD:
         ++check_data_in;
+        /* drop through */
+    case SG_LIB_CAT_UNIT_ATTENTION:
+    case SG_LIB_CAT_SENSE:
     default:
         n = noisy;
         break;
@@ -108,12 +146,12 @@ sg_cmds_process_helper(const char * leadin, int mx_di_len, int resid,
     if (verbose || n) {
         sg_get_sense_str(leadin, sbp, slen, (verbose > 1),
                          sizeof(b), b);
-        fprintf(sg_warnings_strm, "%s", b);
+        pr2ws("%s", b);
         if ((mx_di_len > 0) && (resid > 0)) {
             got = mx_di_len - resid;
             if ((verbose > 2) || check_data_in || (got > 0))
-                fprintf(sg_warnings_strm, "    pass-through requested "
-                        "%d bytes but got %d bytes\n", mx_di_len, got);
+                pr2ws("    pass-through requested %d bytes (data-in) but "
+                      "got %d bytes\n", mx_di_len, got);
         }
     }
     if (o_sense_cat)
@@ -121,13 +159,15 @@ sg_cmds_process_helper(const char * leadin, int mx_di_len, int resid,
     return -2;
 }
 
-/* This is a helper function used by sg_cmds_* implementations after
- * the call to the pass-through. pt_res is returned from do_scsi_pt().
- * If valid sense data is found it is decoded and output to sg_warnings_strm
- * (def: stderr); depending on the 'noisy' and 'verbose' settings.
- * Returns -2 for sense data (may not be fatal), -1 for failed, or the
- * number of data in bytes received. For data out (to device) or no data,
- * set 'mx_di_len' to 0 or less. If -2 returned then sense category
+/* This is a helper function used by sg_cmds_* implementations after the
+ * call to the pass-through. pt_res is returned from do_scsi_pt(). If valid
+ * sense data is found it is decoded and output to sg_warnings_strm (def:
+ * stderr); depending on the 'noisy' and 'verbose' settings. Returns -2 for
+ * "sense" category (may not be fatal), -1 for failed, 0, or a positive
+ * number. If 'mx_di_len > 0' then asks pass-through for resid and returns
+ * (mx_di_len - resid); otherwise returns 0. So for data-in it should return
+ * the actual number of bytes received. For data-out (to device) or no data
+ * call with 'mx_di_len' set to 0 or less. If -2 returned then sense category
  * output via 'o_sense_cat' pointer (if not NULL). Note that several sense
  * categories also have data in bytes received; -2 is still returned. */
 int
@@ -135,25 +175,39 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
                      int pt_res, int mx_di_len, const unsigned char * sbp,
                      int noisy, int verbose, int * o_sense_cat)
 {
-    int got, cat, duration, slen, resid, resp_code;
+    int got, cat, duration, slen, resid, resp_code, sstat;
     char b[1024];
 
     if (NULL == leadin)
         leadin = "";
     if (pt_res < 0) {
+#ifdef SG_LIB_LINUX
+        if (verbose)
+            pr2ws("%s: pass through os error: %s\n", leadin,
+                  safe_strerror(-pt_res));
+        if ((-ENXIO == pt_res) && o_sense_cat) {
+            if (verbose > 2)
+                pr2ws("map ENXIO to SG_LIB_CAT_NOT_READY\n");
+            *o_sense_cat = SG_LIB_CAT_NOT_READY;
+            return -2;
+        } else if (noisy && (0 == verbose))
+            pr2ws("%s: pass through os error: %s\n", leadin,
+                  safe_strerror(-pt_res));
+#else
         if (noisy || verbose)
-            fprintf(sg_warnings_strm, "%s: pass through os error: %s\n",
-                    leadin, safe_strerror(-pt_res));
+            pr2ws("%s: pass through os error: %s\n", leadin,
+                  safe_strerror(-pt_res));
+#endif
         return -1;
     } else if (SCSI_PT_DO_BAD_PARAMS == pt_res) {
-        fprintf(sg_warnings_strm, "%s: bad pass through setup\n", leadin);
+        pr2ws("%s: bad pass through setup\n", leadin);
         return -1;
     } else if (SCSI_PT_DO_TIMEOUT == pt_res) {
-        fprintf(sg_warnings_strm, "%s: pass through timeout\n", leadin);
+        pr2ws("%s: pass through timeout\n", leadin);
         return -1;
     }
     if ((verbose > 2) && ((duration = get_scsi_pt_duration_ms(ptvp)) >= 0))
-        fprintf(sg_warnings_strm, "      duration=%d ms\n", duration);
+        pr2ws("      duration=%d ms\n", duration);
     resid = (mx_di_len > 0) ? get_scsi_pt_resid(ptvp) : 0;
     slen = get_scsi_pt_sense_len(ptvp);
     switch ((cat = get_scsi_pt_result_category(ptvp))) {
@@ -173,18 +227,22 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
         }
         if (mx_di_len > 0) {
             got = mx_di_len - resid;
-            if (verbose && (resid > 0))
-                fprintf(sg_warnings_strm, "    %s: pass-through requested "
-                        "%d bytes but got %d bytes\n", leadin, mx_di_len,
-                        got);
+            if ((verbose > 1) && (resid > 0))
+                pr2ws("    %s: pass-through requested %d bytes (data-in) "
+                      "but got %d bytes\n", leadin, mx_di_len, got);
             return got;
         } else
             return 0;
     case SCSI_PT_RESULT_STATUS: /* other than GOOD and CHECK CONDITION */
+        sstat = get_scsi_pt_status_response(ptvp);
+        if ((SAM_STAT_RESERVATION_CONFLICT == sstat) && o_sense_cat) {
+            /* treat this SCSI status as "sense" category */
+            *o_sense_cat = SG_LIB_CAT_RES_CONFLICT;
+            return -2;
+        }
         if (verbose || noisy) {
-            sg_get_scsi_status_str(get_scsi_pt_status_response(ptvp),
-                                   sizeof(b), b);
-            fprintf(sg_warnings_strm, "%s: scsi status: %s\n", leadin, b);
+            sg_get_scsi_status_str(sstat, sizeof(b), b);
+            pr2ws("%s: scsi status: %s\n", leadin, b);
         }
         return -1;
     case SCSI_PT_RESULT_SENSE:
@@ -193,7 +251,7 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
     case SCSI_PT_RESULT_TRANSPORT_ERR:
         if (verbose || noisy) {
             get_scsi_pt_transport_err_str(ptvp, sizeof(b), b);
-            fprintf(sg_warnings_strm, "%s: transport: %s\n", leadin, b);
+            pr2ws("%s: transport: %s\n", leadin, b);
         }
         if ((SAM_STAT_CHECK_CONDITION == get_scsi_pt_status_response(ptvp))
             && (slen > 0))
@@ -204,25 +262,22 @@ sg_cmds_process_resp(struct sg_pt_base * ptvp, const char * leadin,
     case SCSI_PT_RESULT_OS_ERR:
         if (verbose || noisy) {
             get_scsi_pt_os_err_str(ptvp, sizeof(b), b);
-            fprintf(sg_warnings_strm, "%s: os: %s\n", leadin, b);
+            pr2ws("%s: os: %s\n", leadin, b);
         }
         return -1;
     default:
-        fprintf(sg_warnings_strm, "%s: unknown pass through result "
-                "category (%d)\n", leadin, cat);
+        pr2ws("%s: unknown pass through result category (%d)\n", leadin, cat);
         return -1;
     }
 }
 
-/* Invokes a SCSI INQUIRY command and yields the response
- * Returns 0 when successful, SG_LIB_CAT_INVALID_OP -> not supported,
- * SG_LIB_CAT_ILLEGAL_REQ -> bad field in cdb, SG_LIB_CAT_ABORTED_COMMAND,
- * SG_LIB_CAT_MALFORMED -> bad response, -1 -> other errors */
+/* Invokes a SCSI INQUIRY command and yields the response. Returns 0 when
+ * successful, various SG_LIB_CAT_* positive values or -1 -> other errors */
 int
 sg_ll_inquiry(int sg_fd, int cmddt, int evpd, int pg_op, void * resp,
               int mx_resp_len, int noisy, int verbose)
 {
-    int res, ret, k, sense_cat;
+    int res, ret, k, sense_cat, resid;
     unsigned char inqCmdBlk[INQUIRY_CMDLEN] = {INQUIRY_CMD, 0, 0, 0, 0, 0};
     unsigned char sense_b[SENSE_BUFF_LEN];
     unsigned char * up;
@@ -236,13 +291,11 @@ sg_ll_inquiry(int sg_fd, int cmddt, int evpd, int pg_op, void * resp,
     /* 16 bit allocation length (was 8) is a recent SPC-3 addition */
     inqCmdBlk[3] = (unsigned char)((mx_resp_len >> 8) & 0xff);
     inqCmdBlk[4] = (unsigned char)(mx_resp_len & 0xff);
-    if (NULL == sg_warnings_strm)
-        sg_warnings_strm = stderr;
     if (verbose) {
-        fprintf(sg_warnings_strm, "    inquiry cdb: ");
+        pr2ws("    inquiry cdb: ");
         for (k = 0; k < INQUIRY_CMDLEN; ++k)
-            fprintf(sg_warnings_strm, "%02x ", inqCmdBlk[k]);
-        fprintf(sg_warnings_strm, "\n");
+            pr2ws("%02x ", inqCmdBlk[k]);
+        pr2ws("\n");
     }
     if (resp && (mx_resp_len > 0)) {
         up = (unsigned char *)resp;
@@ -252,7 +305,7 @@ sg_ll_inquiry(int sg_fd, int cmddt, int evpd, int pg_op, void * resp,
     }
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
-        fprintf(sg_warnings_strm, "inquiry: out of memory\n");
+        pr2ws("inquiry: out of memory\n");
         return -1;
     }
     set_scsi_pt_cdb(ptvp, inqCmdBlk, sizeof(inqCmdBlk));
@@ -261,39 +314,42 @@ sg_ll_inquiry(int sg_fd, int cmddt, int evpd, int pg_op, void * resp,
     res = do_scsi_pt(ptvp, sg_fd, DEF_PT_TIMEOUT, verbose);
     ret = sg_cmds_process_resp(ptvp, "inquiry", res, mx_resp_len, sense_b,
                                noisy, verbose, &sense_cat);
+    resid = get_scsi_pt_resid(ptvp);
     destruct_scsi_pt_obj(ptvp);
     if (-1 == ret)
         ;
     else if (-2 == ret) {
         switch (sense_cat) {
-        case SG_LIB_CAT_INVALID_OP:
-        case SG_LIB_CAT_ILLEGAL_REQ:
-        case SG_LIB_CAT_ABORTED_COMMAND:
-            ret = sense_cat;
-            break;
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
             ret = 0;
             break;
         default:
-            ret = -1;
+            ret = sense_cat;
             break;
         }
     } else if (ret < 4) {
         if (verbose)
-            fprintf(sg_warnings_strm, "inquiry: got too few "
-                    "bytes (%d)\n", ret);
+            pr2ws("inquiry: got too few bytes (%d)\n", ret);
         ret = SG_LIB_CAT_MALFORMED;
     } else
         ret = 0;
 
+    if (resid > 0) {
+        if (resid > mx_resp_len) {
+            pr2ws("inquiry: resid (%d) should never exceed requested "
+                  "len=%d\n", resid, mx_resp_len);
+            return ret ? ret : SG_LIB_CAT_MALFORMED;
+        }
+        /* zero unfilled section of response buffer */
+        memset((unsigned char *)resp + (mx_resp_len - resid), 0, resid);
+    }
     return ret;
 }
 
 /* Yields most of first 36 bytes of a standard INQUIRY (evpd==0) response.
- * Returns 0 when successful, SG_LIB_CAT_INVALID_OP -> not supported,
- * SG_LIB_CAT_ILLEGAL_REQ -> bad field in cdb, SG_LIB_CAT_ABORTED_COMMAND,
- * SG_LIB_CAT_MALFORMED -> bad response, -1 -> other errors */
+ * Returns 0 when successful, various SG_LIB_CAT_* positive values or
+ * -1 -> other errors */
 int
 sg_simple_inquiry(int sg_fd, struct sg_simple_inquiry_resp * inq_data,
                   int noisy, int verbose)
@@ -301,7 +357,7 @@ sg_simple_inquiry(int sg_fd, struct sg_simple_inquiry_resp * inq_data,
     int res, ret, k, sense_cat;
     unsigned char inqCmdBlk[INQUIRY_CMDLEN] = {INQUIRY_CMD, 0, 0, 0, 0, 0};
     unsigned char sense_b[SENSE_BUFF_LEN];
-    unsigned char inq_resp[INQUIRY_RESP_INITIAL_LEN];
+    unsigned char inq_resp[SAFE_STD_INQ_RESP_LEN];
     struct sg_pt_base * ptvp;
 
     if (inq_data) {
@@ -310,19 +366,17 @@ sg_simple_inquiry(int sg_fd, struct sg_simple_inquiry_resp * inq_data,
         inq_data->peripheral_type = 0x1f;
     }
     inqCmdBlk[4] = (unsigned char)sizeof(inq_resp);
-    if (NULL == sg_warnings_strm)
-        sg_warnings_strm = stderr;
     if (verbose) {
-        fprintf(sg_warnings_strm, "    inquiry cdb: ");
+        pr2ws("    inquiry cdb: ");
         for (k = 0; k < INQUIRY_CMDLEN; ++k)
-            fprintf(sg_warnings_strm, "%02x ", inqCmdBlk[k]);
-        fprintf(sg_warnings_strm, "\n");
+            pr2ws("%02x ", inqCmdBlk[k]);
+        pr2ws("\n");
     }
     memset(inq_resp, 0, sizeof(inq_resp));
     inq_resp[0] = 0x7f; /* defensive prefill */
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
-        fprintf(sg_warnings_strm, "inquiry: out of memory\n");
+        pr2ws("inquiry: out of memory\n");
         return -1;
     }
     set_scsi_pt_cdb(ptvp, inqCmdBlk, sizeof(inqCmdBlk));
@@ -335,31 +389,25 @@ sg_simple_inquiry(int sg_fd, struct sg_simple_inquiry_resp * inq_data,
         ;
     else if (-2 == ret) {
         switch (sense_cat) {
-        case SG_LIB_CAT_INVALID_OP:
-        case SG_LIB_CAT_ILLEGAL_REQ:
-        case SG_LIB_CAT_ABORTED_COMMAND:
-            ret = sense_cat;
-            break;
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
             ret = 0;
             break;
         default:
-            ret = -1;
+            ret = sense_cat;
             break;
         }
     } else if (ret < 4) {
         if (verbose)
-            fprintf(sg_warnings_strm, "inquiry: got too few "
-                    "bytes (%d)\n", ret);
+            pr2ws("inquiry: got too few bytes (%d)\n", ret);
         ret = SG_LIB_CAT_MALFORMED;
     } else
         ret = 0;
 
-    if (0 == ret) {
+    if (inq_data && (0 == ret)) {
         inq_data->peripheral_qualifier = (inq_resp[0] >> 5) & 0x7;
         inq_data->peripheral_type = inq_resp[0] & 0x1f;
-        inq_data->rmb = (inq_resp[1] & 0x80) ? 1 : 0;
+        inq_data->byte_1 = inq_resp[1];
         inq_data->version = inq_resp[2];
         inq_data->byte_3 = inq_resp[3];
         inq_data->byte_5 = inq_resp[5];
@@ -377,9 +425,8 @@ sg_simple_inquiry(int sg_fd, struct sg_simple_inquiry_resp * inq_data,
  * 'pack_id' is just for diagnostics, safe to set to 0.
  * Looks for progress indicator if 'progress' non-NULL;
  * if found writes value [0..65535] else write -1.
- * Return of 0 -> success, SG_LIB_CAT_UNIT_ATTENTION,
- * SG_LIB_CAT_ABORTED_COMMAND, SG_LIB_CAT_NOT_READY ->
- * device not ready, -1 -> other failure */
+ * Returns 0 when successful, various SG_LIB_CAT_* positive values or
+ * -1 -> other errors */
 int
 sg_ll_test_unit_ready_progress(int sg_fd, int pack_id, int * progress,
                                int noisy, int verbose)
@@ -389,18 +436,16 @@ sg_ll_test_unit_ready_progress(int sg_fd, int pack_id, int * progress,
     unsigned char sense_b[SENSE_BUFF_LEN];
     struct sg_pt_base * ptvp;
 
-    if (NULL == sg_warnings_strm)
-        sg_warnings_strm = stderr;
     if (verbose) {
-        fprintf(sg_warnings_strm, "    test unit ready cdb: ");
+        pr2ws("    test unit ready cdb: ");
         for (k = 0; k < TUR_CMDLEN; ++k)
-            fprintf(sg_warnings_strm, "%02x ", turCmdBlk[k]);
-        fprintf(sg_warnings_strm, "\n");
+            pr2ws("%02x ", turCmdBlk[k]);
+        pr2ws("\n");
     }
 
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
-        fprintf(sg_warnings_strm, "test unit ready: out of memory\n");
+        pr2ws("test unit ready: out of memory\n");
         return -1;
     }
     set_scsi_pt_cdb(ptvp, turCmdBlk, sizeof(turCmdBlk));
@@ -419,19 +464,12 @@ sg_ll_test_unit_ready_progress(int sg_fd, int pack_id, int * progress,
                 *progress = -1;
         }
         switch (sense_cat) {
-        case SG_LIB_CAT_UNIT_ATTENTION:
-        case SG_LIB_CAT_NOT_READY:
-        case SG_LIB_CAT_ABORTED_COMMAND:
-            ret = sense_cat;
-            break;
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
             ret = 0;
             break;
-        case SG_LIB_CAT_INVALID_OP:
-        case SG_LIB_CAT_ILLEGAL_REQ:
         default:
-            ret = -1;
+            ret = sense_cat;
             break;
         }
     } else
@@ -443,9 +481,8 @@ sg_ll_test_unit_ready_progress(int sg_fd, int pack_id, int * progress,
 
 /* Invokes a SCSI TEST UNIT READY command.
  * 'pack_id' is just for diagnostics, safe to set to 0.
- * Return of 0 -> success, SG_LIB_CAT_UNIT_ATTENTION,
- * SG_LIB_CAT_ABORTED_COMMAND, SG_LIB_CAT_NOT_READY ->
- * device not ready, -1 -> other failure */
+ * Returns 0 when successful, various SG_LIB_CAT_* positive values or
+ * -1 -> other errors */
 int
 sg_ll_test_unit_ready(int sg_fd, int pack_id, int noisy, int verbose)
 {
@@ -453,10 +490,8 @@ sg_ll_test_unit_ready(int sg_fd, int pack_id, int noisy, int verbose)
                                           verbose);
 }
 
-/* Invokes a SCSI REQUEST SENSE command. Return of 0 -> success,
- * SG_LIB_CAT_INVALID_OP -> Request Sense not supported??,
- * SG_LIB_CAT_ILLEGAL_REQ -> bad field in cdb,
- * SG_LIB_CAT_ABORTED_COMMAND, -1 -> other failure */
+/* Invokes a SCSI REQUEST SENSE command. Returns 0 when successful, various
+ * SG_LIB_CAT_* positive values or -1 -> other errors */
 int
 sg_ll_request_sense(int sg_fd, int desc, void * resp, int mx_resp_len,
                     int noisy, int verbose)
@@ -469,23 +504,21 @@ sg_ll_request_sense(int sg_fd, int desc, void * resp, int mx_resp_len,
 
     if (desc)
         rsCmdBlk[1] |= 0x1;
-    if (NULL == sg_warnings_strm)
-        sg_warnings_strm = stderr;
     if (mx_resp_len > 0xff) {
-        fprintf(sg_warnings_strm, "mx_resp_len cannot exceed 255\n");
+        pr2ws("mx_resp_len cannot exceed 255\n");
         return -1;
     }
     rsCmdBlk[4] = mx_resp_len & 0xff;
     if (verbose) {
-        fprintf(sg_warnings_strm, "    Request Sense cmd: ");
+        pr2ws("    Request Sense cmd: ");
         for (k = 0; k < REQUEST_SENSE_CMDLEN; ++k)
-            fprintf(sg_warnings_strm, "%02x ", rsCmdBlk[k]);
-        fprintf(sg_warnings_strm, "\n");
+            pr2ws("%02x ", rsCmdBlk[k]);
+        pr2ws("\n");
     }
 
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
-        fprintf(sg_warnings_strm, "request sense: out of memory\n");
+        pr2ws("request sense: out of memory\n");
         return -1;
     }
     set_scsi_pt_cdb(ptvp, rsCmdBlk, sizeof(rsCmdBlk));
@@ -498,25 +531,19 @@ sg_ll_request_sense(int sg_fd, int desc, void * resp, int mx_resp_len,
         ;
     else if (-2 == ret) {
         switch (sense_cat) {
-        case SG_LIB_CAT_INVALID_OP:
-        case SG_LIB_CAT_ILLEGAL_REQ:
-        case SG_LIB_CAT_ABORTED_COMMAND:
-            ret = sense_cat;
-            break;
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
             ret = 0;
             break;
-        case SG_LIB_CAT_NOT_READY:      /* shouldn't happen ?? */
         default:
-            ret = -1;
+            ret = sense_cat;
             break;
         }
     } else {
         if ((mx_resp_len >= 8) && (ret < 8)) {
             if (verbose)
-                fprintf(sg_warnings_strm, "    request sense: got %d "
-                        "bytes in response, too short\n", ret);
+                pr2ws("    request sense: got %d bytes in response, too "
+                      "short\n", ret);
             ret = -1;
         } else
             ret = 0;
@@ -526,10 +553,7 @@ sg_ll_request_sense(int sg_fd, int desc, void * resp, int mx_resp_len,
 }
 
 /* Invokes a SCSI REPORT LUNS command. Return of 0 -> success,
- * SG_LIB_CAT_INVALID_OP -> Report Luns not supported,
- * SG_LIB_CAT_ILLEGAL_REQ -> bad field in cdb,
- * SG_LIB_CAT_ABORTED_COMMAND,
- * SG_LIB_NOT_READY (shouldn't happen), -1 -> other failure */
+ * various SG_LIB_CAT_* positive values or -1 -> other errors */
 int
 sg_ll_report_luns(int sg_fd, int select_report, void * resp, int mx_resp_len,
                   int noisy, int verbose)
@@ -545,18 +569,16 @@ sg_ll_report_luns(int sg_fd, int select_report, void * resp, int mx_resp_len,
     rlCmdBlk[7] = (mx_resp_len >> 16) & 0xff;
     rlCmdBlk[8] = (mx_resp_len >> 8) & 0xff;
     rlCmdBlk[9] = mx_resp_len & 0xff;
-    if (NULL == sg_warnings_strm)
-        sg_warnings_strm = stderr;
     if (verbose) {
-        fprintf(sg_warnings_strm, "    report luns cdb: ");
+        pr2ws("    report luns cdb: ");
         for (k = 0; k < REPORT_LUNS_CMDLEN; ++k)
-            fprintf(sg_warnings_strm, "%02x ", rlCmdBlk[k]);
-        fprintf(sg_warnings_strm, "\n");
+            pr2ws("%02x ", rlCmdBlk[k]);
+        pr2ws("\n");
     }
 
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
-        fprintf(sg_warnings_strm, "report luns: out of memory\n");
+        pr2ws("report luns: out of memory\n");
         return -1;
     }
     set_scsi_pt_cdb(ptvp, rlCmdBlk, sizeof(rlCmdBlk));
@@ -569,18 +591,12 @@ sg_ll_report_luns(int sg_fd, int select_report, void * resp, int mx_resp_len,
         ;
     else if (-2 == ret) {
         switch (sense_cat) {
-        case SG_LIB_CAT_INVALID_OP:
-        case SG_LIB_CAT_ILLEGAL_REQ:
-        case SG_LIB_CAT_ABORTED_COMMAND:
-        case SG_LIB_CAT_NOT_READY:      /* shouldn't happen ?? */
-            ret = sense_cat;
-            break;
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
             ret = 0;
             break;
         default:
-            ret = -1;
+            ret = sense_cat;
             break;
         }
     } else
@@ -588,4 +604,3 @@ sg_ll_report_luns(int sg_fd, int select_report, void * resp, int mx_resp_len,
     destruct_scsi_pt_obj(ptvp);
     return ret;
 }
-
