@@ -1,15 +1,16 @@
 /*
- * Copyright (c) 2005-2013 Douglas Gilbert.
+ * Copyright (c) 2005-2015 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
  */
 
-/* sg_pt_linux version 1.19 20130507 */
+/* sg_pt_linux version 1.24 20150511 */
 
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
@@ -81,6 +82,26 @@ static const char * linux_driver_suggests[] = {
 #define SG_LIB_DRIVER_SENSE    DRIVER_SENSE
 
 
+#ifdef __GNUC__
+static int pr2ws(const char * fmt, ...)
+        __attribute__ ((format (printf, 1, 2)));
+#else
+static int pr2ws(const char * fmt, ...);
+#endif
+
+
+static int
+pr2ws(const char * fmt, ...)
+{
+    va_list args;
+    int n;
+
+    va_start(args, fmt);
+    n = vfprintf(sg_warnings_strm ? sg_warnings_strm : stderr, fmt, args);
+    va_end(args);
+    return n;
+}
+
 
 // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 #if defined(IGNORE_LINUX_BSG) || ! defined(HAVE_LINUX_BSG_H)
@@ -122,10 +143,7 @@ scsi_pt_open_flags(const char * device_name, int flags, int verbose)
     int fd;
 
     if (verbose > 1) {
-        if (NULL == sg_warnings_strm)
-            sg_warnings_strm = stderr;
-        fprintf(sg_warnings_strm, "open %s with flags=0x%x\n", device_name,
-                flags);
+        pr2ws("open %s with flags=0x%x\n", device_name, flags);
     }
     fd = open(device_name, flags);
     if (fd < 0)
@@ -151,6 +169,12 @@ construct_scsi_pt_obj()
 {
     struct sg_pt_linux_scsi * ptp;
 
+    /* The following 2 lines are temporary. It is to avoid a NULL pointer
+     * crash when an old utility is used with a newer library built after
+     * the sg_warnings_strm cleanup */
+    if (NULL == sg_warnings_strm)
+        sg_warnings_strm = stderr;
+
     ptp = (struct sg_pt_linux_scsi *)
           calloc(1, sizeof(struct sg_pt_linux_scsi));
     if (ptp) {
@@ -252,7 +276,7 @@ set_scsi_pt_tag(struct sg_pt_base * vp, uint64_t tag)
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
     ++ptp->in_err;
-    tag = tag;                  /* dummy to silence compiler */
+    if (tag) { ; }     /* unused, suppress warning */
 }
 
 /* Note that task management function codes are transport specific */
@@ -262,7 +286,7 @@ set_scsi_pt_task_management(struct sg_pt_base * vp, int tmf_code)
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
     ++ptp->in_err;
-    tmf_code = tmf_code;        /* dummy to silence compiler */
+    if (tmf_code) { ; }     /* unused, suppress warning */
 }
 
 void
@@ -271,8 +295,8 @@ set_scsi_pt_task_attr(struct sg_pt_base * vp, int attribute, int priority)
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
     ++ptp->in_err;
-    attribute = attribute;      /* dummy to silence compiler */
-    priority = priority;        /* dummy to silence compiler */
+    if (attribute) { ; }     /* unused, suppress warning */
+    if (priority) { ; }      /* unused, suppress warning */
 }
 
 #ifndef SG_FLAG_Q_AT_TAIL
@@ -287,15 +311,14 @@ set_scsi_pt_flags(struct sg_pt_base * vp, int flags)
 {
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    /* default action of SG (v3) is QUEUE_AT_HEAD */
+    /* default action of sg driver [sg v3 interface] is QUEUE_AT_HEAD */
     /* default action of block layer SG_IO ioctl is QUEUE_AT_TAIL */
-    if (SCSI_PT_FLAGS_QUEUE_AT_TAIL & flags) {
-        ptp->io_hdr.flags |= SG_FLAG_Q_AT_TAIL;
-        ptp->io_hdr.flags &= ~SG_FLAG_Q_AT_HEAD;
-    }
-    if (SCSI_PT_FLAGS_QUEUE_AT_HEAD & flags) {
+    if (SCSI_PT_FLAGS_QUEUE_AT_HEAD & flags) {  /* favour AT_HEAD */
         ptp->io_hdr.flags |= SG_FLAG_Q_AT_HEAD;
         ptp->io_hdr.flags &= ~SG_FLAG_Q_AT_TAIL;
+    } else if (SCSI_PT_FLAGS_QUEUE_AT_TAIL & flags) {
+        ptp->io_hdr.flags |= SG_FLAG_Q_AT_TAIL;
+        ptp->io_hdr.flags &= ~SG_FLAG_Q_AT_HEAD;
     }
 }
 
@@ -307,18 +330,15 @@ do_scsi_pt(struct sg_pt_base * vp, int fd, int time_secs, int verbose)
 {
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    if (NULL == sg_warnings_strm)
-        sg_warnings_strm = stderr;
     ptp->os_err = 0;
     if (ptp->in_err) {
         if (verbose)
-            fprintf(sg_warnings_strm, "Replicated or unused set_scsi_pt... "
-                    "functions\n");
+            pr2ws("Replicated or unused set_scsi_pt... functions\n");
         return SCSI_PT_DO_BAD_PARAMS;
     }
     if (NULL == ptp->io_hdr.cmdp) {
         if (verbose)
-            fprintf(sg_warnings_strm, "No SCSI command (cdb) given\n");
+            pr2ws("No SCSI command (cdb) given\n");
         return SCSI_PT_DO_BAD_PARAMS;
     }
     /* io_hdr.timeout is in milliseconds */
@@ -329,8 +349,8 @@ do_scsi_pt(struct sg_pt_base * vp, int fd, int time_secs, int verbose)
     if (ioctl(fd, SG_IO, &ptp->io_hdr) < 0) {
         ptp->os_err = errno;
         if (verbose > 1)
-            fprintf(sg_warnings_strm, "ioctl(SG_IO) failed: %s (errno=%d)\n",
-                    strerror(ptp->os_err), ptp->os_err);
+            pr2ws("ioctl(SG_IO) failed: %s (errno=%d)\n",
+                  strerror(ptp->os_err), ptp->os_err);
         return -ptp->os_err;
     }
     return 0;
@@ -532,20 +552,17 @@ find_bsg_major(int verbose)
     int n;
 
     if (NULL == (fp = fopen(proc_devices, "r"))) {
-        if (NULL == sg_warnings_strm)
-            sg_warnings_strm = stderr;
         if (verbose)
-            fprintf(sg_warnings_strm, "fopen %s failed: %s\n", proc_devices,
-                    strerror(errno));
+            pr2ws("fopen %s failed: %s\n", proc_devices, strerror(errno));
         return;
     }
     while ((cp = fgets(b, sizeof(b), fp))) {
-        if ((1 == sscanf(b, "%s", a)) &&
+        if ((1 == sscanf(b, "%126s", a)) &&
             (0 == memcmp(a, "Character", 9)))
             break;
     }
     while (cp && (cp = fgets(b, sizeof(b), fp))) {
-        if (2 == sscanf(b, "%d %s", &n, a)) {
+        if (2 == sscanf(b, "%d %126s", &n, a)) {
             if (0 == strcmp("bsg", a)) {
                 bsg_major = n;
                 break;
@@ -554,13 +571,10 @@ find_bsg_major(int verbose)
             break;
     }
     if (verbose > 3) {
-        if (NULL == sg_warnings_strm)
-            sg_warnings_strm = stderr;
         if (cp)
-            fprintf(sg_warnings_strm, "found bsg_major=%d\n", bsg_major);
+            pr2ws("found bsg_major=%d\n", bsg_major);
         else
-            fprintf(sg_warnings_strm, "found no bsg char device in %s\n",
-                proc_devices);
+            pr2ws("found no bsg char device in %s\n", proc_devices);
     }
     fclose(fp);
 }
@@ -588,12 +602,8 @@ scsi_pt_open_flags(const char * device_name, int flags, int verbose)
         bsg_major_checked = 1;
         find_bsg_major(verbose);
     }
-    if (verbose > 1) {
-        if (NULL == sg_warnings_strm)
-            sg_warnings_strm = stderr;
-        fprintf(sg_warnings_strm, "open %s with flags=0x%x\n", device_name,
-                flags);
-    }
+    if (verbose > 1)
+        pr2ws("open %s with flags=0x%x\n", device_name, flags);
     fd = open(device_name, flags);
     if (fd < 0)
         fd = -errno;
@@ -754,17 +764,32 @@ set_scsi_pt_task_attr(struct sg_pt_base * vp, int attribute, int priority)
 #ifndef BSG_FLAG_Q_AT_TAIL
 #define BSG_FLAG_Q_AT_TAIL 0x10
 #endif
+#ifndef BSG_FLAG_Q_AT_HEAD
+#define BSG_FLAG_Q_AT_HEAD 0x20
+#endif
+
+/* Need this later if translated to v3 interface */
+#ifndef SG_FLAG_Q_AT_TAIL
+#define SG_FLAG_Q_AT_TAIL 0x10
+#endif
+#ifndef SG_FLAG_Q_AT_HEAD
+#define SG_FLAG_Q_AT_HEAD 0x20
+#endif
 
 void
 set_scsi_pt_flags(struct sg_pt_base * vp, int flags)
 {
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    /* default action of bsg (sg v4) is QUEUE_AT_HEAD */
-    if (SCSI_PT_FLAGS_QUEUE_AT_TAIL & flags)
-        ptp->io_hdr.flags |= BSG_FLAG_Q_AT_TAIL;
-    if (SCSI_PT_FLAGS_QUEUE_AT_HEAD & flags)
+    /* default action of bsg driver (sg v4) is QUEUE_AT_HEAD */
+    /* default action of block layer SG_IO ioctl is QUEUE_AT_TAIL */
+    if (SCSI_PT_FLAGS_QUEUE_AT_HEAD & flags) {  /* favour AT_HEAD */
+        ptp->io_hdr.flags |= BSG_FLAG_Q_AT_HEAD;
         ptp->io_hdr.flags &= ~BSG_FLAG_Q_AT_TAIL;
+    } else if (SCSI_PT_FLAGS_QUEUE_AT_TAIL & flags) {
+        ptp->io_hdr.flags |= BSG_FLAG_Q_AT_TAIL;
+        ptp->io_hdr.flags &= ~BSG_FLAG_Q_AT_HEAD;
+    }
 }
 
 /* N.B. Returns din_resid and ignores dout_resid */
@@ -915,7 +940,7 @@ do_scsi_pt_v3(struct sg_pt_linux_scsi * ptp, int fd, int time_secs,
     if (ptp->io_hdr.din_xfer_len > 0) {
         if (ptp->io_hdr.dout_xfer_len > 0) {
             if (verbose)
-                fprintf(sg_warnings_strm, "sgv3 doesn't support bidi\n");
+                pr2ws("sgv3 doesn't support bidi\n");
             return SCSI_PT_DO_BAD_PARAMS;
         }
         v3_hdr.dxferp = (void *)(long)ptp->io_hdr.din_xferp;
@@ -931,10 +956,14 @@ do_scsi_pt_v3(struct sg_pt_linux_scsi * ptp, int fd, int time_secs,
         v3_hdr.mx_sb_len = (unsigned char)ptp->io_hdr.max_response_len;
     }
     v3_hdr.pack_id = (int)ptp->io_hdr.spare_in;
+    if (BSG_FLAG_Q_AT_HEAD & ptp->io_hdr.flags)
+        v3_hdr.flags |= SG_FLAG_Q_AT_HEAD;      /* favour AT_HEAD */
+    else if (BSG_FLAG_Q_AT_TAIL & ptp->io_hdr.flags)
+        v3_hdr.flags |= SG_FLAG_Q_AT_TAIL;
 
     if (NULL == v3_hdr.cmdp) {
         if (verbose)
-            fprintf(sg_warnings_strm, "No SCSI command (cdb) given\n");
+            pr2ws("No SCSI command (cdb) given\n");
         return SCSI_PT_DO_BAD_PARAMS;
     }
     /* io_hdr.timeout is in milliseconds, if greater than zero */
@@ -943,8 +972,8 @@ do_scsi_pt_v3(struct sg_pt_linux_scsi * ptp, int fd, int time_secs,
     if (ioctl(fd, SG_IO, &v3_hdr) < 0) {
         ptp->os_err = errno;
         if (verbose > 1)
-            fprintf(sg_warnings_strm, "ioctl(SG_IO v3) failed: %s "
-                    "(errno=%d)\n", strerror(ptp->os_err), ptp->os_err);
+            pr2ws("ioctl(SG_IO v3) failed: %s (errno=%d)\n",
+                  strerror(ptp->os_err), ptp->os_err);
         return -ptp->os_err;
     }
     ptp->io_hdr.device_status = (__u32)v3_hdr.status;
@@ -969,13 +998,10 @@ do_scsi_pt(struct sg_pt_base * vp, int fd, int time_secs, int verbose)
         bsg_major_checked = 1;
         find_bsg_major(verbose);
     }
-    if (NULL == sg_warnings_strm)
-        sg_warnings_strm = stderr;
     ptp->os_err = 0;
     if (ptp->in_err) {
         if (verbose)
-            fprintf(sg_warnings_strm, "Replicated or unused set_scsi_pt... "
-                    "functions\n");
+            pr2ws("Replicated or unused set_scsi_pt... functions\n");
         return SCSI_PT_DO_BAD_PARAMS;
     }
     if (bsg_major <= 0)
@@ -986,8 +1012,8 @@ do_scsi_pt(struct sg_pt_base * vp, int fd, int time_secs, int verbose)
         if (fstat(fd, &a_stat) < 0) {
             ptp->os_err = errno;
             if (verbose > 1)
-                fprintf(sg_warnings_strm, "fstat() failed: %s (errno=%d)\n",
-                        strerror(ptp->os_err), ptp->os_err);
+                pr2ws("fstat() failed: %s (errno=%d)\n",
+                      strerror(ptp->os_err), ptp->os_err);
             return -ptp->os_err;
         }
         if (! S_ISCHR(a_stat.st_mode) ||
@@ -997,7 +1023,7 @@ do_scsi_pt(struct sg_pt_base * vp, int fd, int time_secs, int verbose)
 
     if (! ptp->io_hdr.request) {
         if (verbose)
-            fprintf(sg_warnings_strm, "No SCSI command (cdb) given (v4)\n");
+            pr2ws("No SCSI command (cdb) given (v4)\n");
         return SCSI_PT_DO_BAD_PARAMS;
     }
     /* io_hdr.timeout is in milliseconds */
@@ -1015,8 +1041,8 @@ do_scsi_pt(struct sg_pt_base * vp, int fd, int time_secs, int verbose)
     if (ioctl(fd, SG_IO, &ptp->io_hdr) < 0) {
         ptp->os_err = errno;
         if (verbose > 1)
-            fprintf(sg_warnings_strm, "ioctl(SG_IO v4) failed: %s "
-                    "(errno=%d)\n", strerror(ptp->os_err), ptp->os_err);
+            pr2ws("ioctl(SG_IO v4) failed: %s (errno=%d)\n",
+                  strerror(ptp->os_err), ptp->os_err);
         return -ptp->os_err;
     }
     return 0;
