@@ -2,7 +2,7 @@
 # Skript to rescan SCSI bus, using the 
 # scsi add-single-device mechanism
 # (c) 1998--2010 Kurt Garloff <kurt@garloff.de>, GNU GPL v2 or v3
-# (c) 2006--2013 Hannes Reinecke, GNU GPL v2 or later
+# (c) 2006--2014 Hannes Reinecke, GNU GPL v2 or later
 # $Id: rescan-scsi-bus.sh,v 1.57 2012/03/31 14:08:48 garloff Exp $
 
 SCAN_WILD_CARD=4294967295
@@ -91,8 +91,8 @@ findhosts ()
       num=$name
       driverinfo=$driver
       if test -r $hostdir/status; then
-	num=$(printf '%d\n' `sed -n 's/SCSI host number://p' $hostdir/status`)
-	driverinfo="$driver:$name"
+        num=$(printf '%d\n' `sed -n 's/SCSI host number://p' $hostdir/status`)
+        driverinfo="$driver:$name"
       fi
       hosts="$hosts $num"
       echo "Host adapter $num ($driverinfo) found."
@@ -105,22 +105,22 @@ printtype ()
   local type=$1
 
   case "$type" in
-    0) echo "Direct-Access    " ;;
+    0) echo "Direct-Access" ;;
     1) echo "Sequential-Access" ;;
-    2) echo "Printer          " ;;
-    3) echo "Processor        " ;;
-    4) echo "WORM             " ;;
-    5) echo "CD-ROM           " ;;
-    6) echo "Scanner          " ;;
-    7) echo "Optical Device   " ;;
-    8) echo "Medium Changer   " ;;
-    9) echo "Communications   " ;;
-    10) echo "Unknown          " ;;
-    11) echo "Unknown          " ;;
-    12) echo "RAID             " ;;
-    13) echo "Enclosure        " ;;
+    2) echo "Printer" ;;
+    3) echo "Processor" ;;
+    4) echo "WORM" ;;
+    5) echo "CD-ROM" ;;
+    6) echo "Scanner" ;;
+    7) echo "Optical Device" ;;
+    8) echo "Medium Changer" ;;
+    9) echo "Communications" ;;
+    10) echo "Unknown" ;;
+    11) echo "Unknown" ;;
+    12) echo "RAID" ;;
+    13) echo "Enclosure" ;;
     14) echo "Direct-Access-RBC" ;;
-    *) echo "Unknown          " ;;
+    *) echo "Unknown" ;;
   esac
 }
 
@@ -147,19 +147,19 @@ procscsiscsi ()
     if [ -d  "$SCSIPATH" ] ; then
       SCSISTR="Host: scsi${host} Channel: $CHANNEL Id: $ID Lun: $LUN"
       if [ "$LN" -gt 0 ] ; then
-	IVEND=$(cat ${SCSIPATH}/device/vendor)
-	IPROD=$(cat ${SCSIPATH}/device/model)
-	IPREV=$(cat ${SCSIPATH}/device/rev)
-	SCSIDEV=$(printf '  Vendor: %-08s Model: %-16s Rev: %-4s' "$IVEND" "$IPROD" "$IPREV")
-	SCSISTR="$SCSISTR
+        IVEND=$(cat ${SCSIPATH}/device/vendor)
+        IPROD=$(cat ${SCSIPATH}/device/model)
+        IPREV=$(cat ${SCSIPATH}/device/rev)
+        SCSIDEV=$(printf '  Vendor: %-08s Model: %-16s Rev: %-4s' "$IVEND" "$IPROD" "$IPREV")
+        SCSISTR="$SCSISTR
 $SCSIDEV"
       fi
       if [ "$LN" -gt 1 ] ; then
-	ILVL=$(cat ${SCSIPATH}/device/scsi_level)
-	type=$(cat ${SCSIPATH}/device/type)
-	ITYPE=$(printtype $type)
-	SCSITMP=$(printf '  Type:   %-16s                ANSI SCSI revision: %02d' "$ITYPE" "$((ILVL - 1))")
-	SCSISTR="$SCSISTR
+        ILVL=$(cat ${SCSIPATH}/device/scsi_level)
+        type=$(cat ${SCSIPATH}/device/type)
+        ITYPE=$(printtype $type)
+        SCSITMP=$(printf '  Type:   %-17s                ANSI SCSI revision: %02d' "$ITYPE" "$((ILVL - 1))")
+        SCSISTR="$SCSISTR
 $SCSITMP"
       fi
     else
@@ -175,14 +175,14 @@ $SCSITMP"
 # Find sg device with 2.6 sysfs support
 sgdevice26 ()
 {
-  if test -e /sys/class/scsi_device/$host\:$channel\:$id\:$lun/device/generic; then	
+  if test -e /sys/class/scsi_device/$host\:$channel\:$id\:$lun/device/generic; then
     SGDEV=`readlink /sys/class/scsi_device/$host\:$channel\:$id\:$lun/device/generic`
     SGDEV=`basename $SGDEV`
   else
     for SGDEV in /sys/class/scsi_generic/sg*; do
       DEV=`readlink $SGDEV/device`
       if test "${DEV##*/}" = "$host:$channel:$id:$lun"; then
-	SGDEV=`basename $SGDEV`; return
+        SGDEV=`basename $SGDEV`; return
       fi
     done
     SGDEV=""
@@ -231,6 +231,9 @@ testonline ()
 {
   : testonline
   RC=0
+  # Set default values
+  IPTYPE=31
+  IPQUAL=3
   if test ! -x /usr/bin/sg_turs; then return 0; fi
   sgdevice
   if test -z "$SGDEV"; then return 0; fi
@@ -256,6 +259,10 @@ testonline ()
   RC=0
   # OK, device online, compare INQUIRY string
   INQ=`sg_inq $sg_len_arg /dev/$SGDEV 2>/dev/null`
+  if [ -z "$INQ" ] ; then
+    echo -e "\e[A\e[A\e[A\e[A${red}$SGDEV changed: ${bold}INQUIRY failed${norm}    \n\n\n"
+    return 2
+  fi
   IVEND=`echo "$INQ" | grep 'Vendor identification:' | sed 's/^[^:]*: \(.*\)$/\1/'`
   IPROD=`echo "$INQ" | grep 'Product identification:' | sed 's/^[^:]*: \(.*\)$/\1/'`
   IPREV=`echo "$INQ" | grep 'Product revision level:' | sed 's/^[^:]*: \(.*\)$/\1/'`
@@ -263,6 +270,8 @@ testonline ()
   IPTYPE=`echo "$INQ" | sed -n 's/.* Device_type=\([0-9]*\) .*/\1/p'`
   IPQUAL=`echo "$INQ" | sed -n 's/ *PQual=\([0-9]*\)  Device.*/\1/p'`
   if [ "$IPQUAL" != 0 ] ; then
+    [ -z "$IPQUAL" ] && IPQUAL=3
+    [ -z "$IPTYPE" ] && IPTYPE=31
     echo -e "\e[A\e[A\e[A\e[A${red}$SGDEV changed: ${bold}LU not available (PQual $IPQUAL)${norm}    \n\n\n"
     return 2
   fi
@@ -277,8 +286,8 @@ testonline ()
     echo -e "\e[A\e[A\e[A\e[A${red}$SGDEV changed: ${bold}\nfrom:${SCSISTR#* } \nto: $STR ${norm} \n\n\n"
     return 1
   fi
-  TMPSTR=`echo "$SCSISTR" | sed -n 's/.*Type: *\(.*\) *ANSI.*/\1/p'`
-  if [ $TMPSTR != $TYPE ] ; then
+  TMPSTR=`echo "$SCSISTR" | sed -n 's/.*Type: *\(.*\) *ANSI.*/\1/p' | sed 's/ *$//g'`
+  if [ "$TMPSTR" != "$TYPE" ] ; then
     echo -e "\e[A\e[A\e[A\e[A${red}$SGDEV changed: ${bold}\nfrom:${TMPSTR} \nto: $TYPE ${norm} \n\n\n"
     return 1
   fi
@@ -313,7 +322,7 @@ chanlist ()
     chan=${cil%%:*}
     for tmpchan in $channelsearch ; do
       if test "$chan" -eq $tmpchan ; then
-	chan=
+        chan=
       fi
     done
     if test -n "$chan" ; then
@@ -339,8 +348,8 @@ idlist ()
     newid=$id
     for tmpid in $idsearch ; do
       if test $id -eq $tmpid ; then
-	newid=
-	break
+        newid=
+        break
       fi
     done
     if test -n "$newid" ; then
@@ -385,9 +394,17 @@ getluns()
 # Wait for udev to settle (create device nodes etc.)
 udevadm_settle()
 {
+  local tmo=60
   if test -x /sbin/udevadm; then 
     print_and_scroll_back " Calling udevadm settle (can take a while) "
-    /sbin/udevadm settle
+    # Loop for up to 60 seconds if sd devices still are settling..
+    # This allows us to continue if udev events are stuck on multipaths in recovery mode
+    while [ $tmo -gt 0 ] ; do
+      if ! /sbin/udevadm settle --timeout=1 | egrep -q sd[a-z]+ ; then
+        break;
+      fi
+      let tmo=$tmo-1
+    done
     white_out
   elif test -x /sbin/udevsettle; then
     print_and_scroll_back " Calling udevsettle (can take a while) "
@@ -456,13 +473,13 @@ dolunscan()
           incrrmvd "$host:$channel:$id:$lun"
         fi
         echo 1 > /sys/class/scsi_device/${host}:${channel}:${id}:${lun}/device/delete
-	usleep 20000
+        usleep 20000
       else
         echo "scsi remove-single-device $devnr" > /proc/scsi/scsi
-	if test $RC -eq 1 -o $lun -eq 0 ; then
+        if test $RC -eq 1 -o $lun -eq 0 ; then
           # Try readding, should fail if device is gone
           echo "scsi add-single-device $devnr" > /proc/scsi/scsi
-	fi
+        fi
       fi
     fi
     if test $RC = 0 -o "$forcerescan" ; then
@@ -534,7 +551,7 @@ doreportlun()
       # Find alternative LUN to send getluns to
       for dev in /sys/class/scsi_device/${host}:${channel}:${id}:*; do
         [ -d "$dev" ] || continue
-	lun=${dev##*:}
+        lun=${dev##*:}
         break
       done
     fi
@@ -572,10 +589,10 @@ doreportlun()
     # OK, is existing $lun (still) in reported list
     for tmplun in $targetluns; do
       if test $tmplun -eq $lun ; then
-	inlist=1
-	dolunscan $lun0added
+        inlist=1
+        dolunscan $lun0added
       else
-	newsearch="$newsearch $tmplun"
+        newsearch="$newsearch $tmplun"
       fi
     done
     # OK, we have now done a lunscan on $lun and 
@@ -605,9 +622,9 @@ dosearch ()
     fi
     for id in $idsearch; do
       if test -z "$lunsearch" ; then
-	doreportlun
+        doreportlun
       else
-	for lun in $lunsearch; do
+        for lun in $lunsearch; do
           dolunscan
         done
       fi
@@ -652,12 +669,12 @@ searchexisting()
     id=`echo $target | cut -d":" -f 3`
     if [ -n "$channelsearch" ] ; then
       for tmpch in $channelsearch ; do
-        test $tmpch -eq $channel && match=1	
+        test $tmpch -eq $channel && match=1
       done
     else
       match=1
     fi
-		
+
     test $match -eq 0 && continue
     match=0
 
@@ -683,6 +700,8 @@ findremapped()
   local id_serial=
   local id_serial_old=
   local sysfs_devpath=
+  local mpath_uuid=
+  local remapped=
   mpaths=""
   local tmpfile="/tmp/rescan-scsi-bus-`date +s`"
 
@@ -691,7 +710,7 @@ findremapped()
   # Get all of the ID_SERIAL attributes, after finding their sd node
   for hctl in $devs ; do
     if [ -d /sys/class/scsi_device/$hctl/device/block ] ; then
-      sddev=`ls /sys/class/scsi_device/$hctl/device/block`	
+      sddev=`ls /sys/class/scsi_device/$hctl/device/block`
       id_serial_old=`udevadm info -q all -n $sddev | grep "ID_SERIAL=" | cut -d"=" -f2`
       [ -z "$id_serial_old" ] && id_serial_old="none"
       echo "$hctl $sddev $id_serial_old" >> $tmpfile
@@ -701,33 +720,46 @@ findremapped()
   # Trigger udev to update the info
   echo -n "Triggering udev to update device information... "
   /sbin/udevadm trigger
-  udevadm_settle 2&>1 /dev/null
+  udevadm_settle 2>&1 /dev/null
   echo "Done"
 
   # See what changed and reload the respective multipath device if applicable
   while read hctl sddev id_serial_old ; do
+    remapped=0
     id_serial=`udevadm info -q all -n $sddev | grep "ID_SERIAL=" | cut -d"=" -f2`
     [ -z "$id_serial" ] && id_serial="none"
     if [ "$id_serial_old" != "$id_serial" ] ; then
-      if [ "$id_serial" = "1" ] ; then
-        continue # the lun was unmapped and is getting the blank scsi id (1)
-      fi
-      printf "${yellow}REMAPPED: $norm"
-      host=`echo $hctl | cut -d":" -f1`
-      channel=`echo $hctl | cut -d":" -f2`
-      id=`echo $hctl | cut -d":" -f3`
-      lun=`echo $hctl | cut -d":" -f4`
-      procscsiscsi
-      echo "$SCSISTR"
-      incrchgd "$hctl"
+      remapped=1
     fi
+    # If udev events updated the disks already, but the multipath device isn't update
+    # check for old devices to make sure we found remapped luns
+    if [ -n "$mp_enable" ] && [ $remapped -eq 0 ]; then
+      findmultipath $sddev $id_serial
+      if [ $? -eq 1 ] ; then
+        remapped=1
+      fi
+    fi
+
+    # if uuid is 1, it's unmapped, so we don't want to treat it as a remap
+    # if remapped flag is 0, just skip the rest of the logic
+    if [ "$id_serial" = "1" ] || [ $remapped -eq 0 ] ; then
+      continue
+    fi
+    printf "${yellow}REMAPPED: $norm"
+    host=`echo $hctl | cut -d":" -f1`
+    channel=`echo $hctl | cut -d":" -f2`
+    id=`echo $hctl | cut -d":" -f3`
+    lun=`echo $hctl | cut -d":" -f4`
+    procscsiscsi
+    echo "$SCSISTR"
+    incrchgd "$hctl"
   done < $tmpfile
-  rm $tmpfile 2&>1 /dev/null
+  [ -f $tmpfile ] && rm $tmpfile
 
   if test -n "$mp_enable" -a -n "$mpaths" ; then
     echo "Updating multipath device mappings"
     flushmpaths
-    $MULTIPATH | grep "create:" 2> /dev/null #2&>1 /dev/null
+    $MULTIPATH | grep "create:" 2> /dev/null
   fi
 }
 
@@ -746,8 +778,10 @@ incrchgd()
 {
   local hctl="$1"
   if test -n "$hctl" ; then
-    let updated+=1
-    CHGDEVS="$CHGDEVS\t[$hctl]\n"
+    if ! echo $CHGDEVS | grep -q "\[$hctl\]"; then
+      let updated+=1
+      CHGDEVS="$CHGDEVS\t[$hctl]\n"
+    fi
   else
     return
   fi
@@ -793,30 +827,61 @@ findsddev()
   return 0
 }
 
+addmpathtolist()
+{
+  local mp="$1"
+  local mp2=
+
+  for mp2 in $mpaths ; do
+    # The multipath device is already in the list
+    if [ "$mp2" = "$mp" ] ; then
+      return
+    fi
+  done
+  mpaths="$mpaths $mp"
+}
+
 findmultipath()
 {
   local dev="$1"
+  local find_mismatch="$2"
   local mp=
   local mp2=
+  local found_dup=0
 
   # Need a sdev, and executable multipath and dmsetup command here
-  if [ -z "$dev" ] || [ ! -x $DMSETUP ] ; then
+  if [ -z "$dev" ] || [ ! -x $DMSETUP ] || [ ! -x $MULTIPATH ] ; then
     return 1
   fi
 
   local maj_min=`cat /sys/block/$dev/dev`
   for mp in $($DMSETUP ls --target=multipath | cut -f 1) ; do
     if $($DMSETUP status $mp | grep -q " $maj_min ") ; then
-      for mp2 in $mpaths ; do
-        # mp device is already there, return
-        if [ "$mp2" = "$mp" ] ; then
-          return
+      # With two arguments, look up current uuid from sysfs
+      # if it doesn't match what was passed, this multipath
+      # device is not updated, so this is a remapped LUN
+      if [ -n "$find_mismatch" ] ; then
+        mp2=`$MULTIPATH -l $mp | egrep -o dm-[0-9]+`
+        mp2=`cat /sys/block/$mp2/dm/uuid | cut -f2 -d-`
+        if [ "$find_mismatch" != "$mp2" ] ; then
+          addmpathtolist $mp
+          found_dup=1
         fi
-      done
-      mpaths="$mpaths $mp"
+        continue
+      fi
+      # Normal mode: Find the first multipath with the sdev
+      # and add it to the list
+      addmpathtolist $mp
       return
     fi
   done
+
+  # Return 1 to signal that a duplicate was found to the calling function
+  if [ $found_dup -eq 1 ] ; then
+    return 1
+  else
+    return 0
+  fi
 }
 
 reloadmpaths()
@@ -827,51 +892,58 @@ reloadmpaths()
     return
   fi
 
+  # Pass 1 as the argument to reload all mpaths
   if [ "$1" = "1" ] ; then
     echo "Reloading all multipath devices"
-    $MULTIPATH -r 2&>1 /dev/null
+    $MULTIPATH -r > /dev/null 2>&1
     return
   fi
 
   # Reload the multipath devices
   for mpath in $mpaths ; do
-    echo "Reloading multipath device $mpath"
-    $MULTIPATH -r $mpath 2&>1 /dev/null
+    echo -n "Reloading multipath device $mpath... "
+    $MULTIPATH -r $mpath > /dev/null 2>&1
+    if test "$?" = "0" ; then
+      echo "Done"
+    else
+      echo "Fail"
+    fi
   done
 }
 
 flushmpaths()
 {
   local mpath
-  # Mode: flush only failed
+  local remove=""
+  local i
+  local flush_retries=5
+
   if test -n "$1" ; then
     for mpath in $($DMSETUP ls --target=multipath | cut -f 1) ; do
       num=$($DMSETUP status $mpath | awk 'BEGIN{RS=" ";active=0}/[0-9]+:[0-9]+/{dev=1}/A/{if (dev == 1) active++; dev=0} END{ print active }')
       if [ $num -eq 0 ] ; then
-        echo -n "Flushing multipath device $mpath...  "
-        $DMSETUP message $mpath 0 fail_if_no_path 2&>1 /dev/null
-        $MULTIPATH -f $mpath 2&>1 /dev/null
-        $DMSETUP status $mpath 2&>1 /dev/null
-        if test "$?" = "1" ; then
-          echo "Done"
-        else
-          echo "Fail"
-        fi
+        remove="$remove $mpath"
       fi
     done
-    return
+  else
+    remove="$mpaths"
   fi
-  # Flush all the devs specified in $mpaths
-  for mpath in $mpaths ; do
-    echo -n "Flushing multipath device $mpath...  "
-    $DMSETUP message $mpath 0 fail_if_no_path 2&>1 /dev/null
-    $MULTIPATH -f $mpath 2&>1 /dev/null
-    $DMSETUP status $mpath 2&>1 /dev/null
-    if test "$?" = "1" ; then
-      echo "Done"
-    else
-      echo "Fail"
-    fi
+
+  for mpath in $remove ; do
+    i=0
+    echo -n "Flushing multipath device $mpath... "
+    while [ $i -lt $flush_retries ] ; do
+      $DMSETUP message $mpath 0 fail_if_no_path > /dev/null 2>&1
+      $MULTIPATH -f $mpath > /dev/null 2>&1
+      if test "$?" = "0" ; then
+        echo "Done ($i retries)"
+        break
+      elif test $i -eq $flush_retries ; then
+        echo "Fail"
+      fi
+      usleep 20000
+      let i=$i+1
+    done
   done
 }
 
@@ -884,16 +956,20 @@ findresized()
   local new_size=
   local sysfs_path=
   local sddev=
+  local i=
+  local m=
+  local mpathsize=
+  declare -a mpathsizes
 
   for hctl in $devs ; do
     sysfs_path="/sys/class/scsi_device/$hctl/device"
     if [ -d "$sysfs_path/block" ] ; then
       sddev=`ls $sysfs_path/block`
       size=`cat $sysfs_path/block/$sddev/size`
-	
+
       echo 1 > $sysfs_path/rescan
-      new_size=`cat $sysfs_path/block/$sddev/size`	
-		
+      new_size=`cat $sysfs_path/block/$sddev/size`
+
       if [ "$size" != "$new_size" ] && [ "$size" != "0" ] && [ "$new_size" != "0" ] ; then
         printf "${yellow}RESIZED: $norm"
         host=`echo $hctl | cut -d":" -f1`
@@ -909,7 +985,18 @@ findresized()
   done
 
   if test -n "$mp_enable" -a -n "$mpaths" ; then
+    i=0
+    for m in $mpaths ; do
+      mpathsizes[$i]="`$MULTIPATH -l $m | egrep -o [0-9]+.[0-9]+[KMGT]`"
+      let i=$i+1
+    done
     reloadmpaths
+    i=0
+    for m in $mpaths ; do
+      mpathsize="`$MULTIPATH -l $m | egrep -o [0-9\.]+[KMGT]`"
+      echo "$m ${mpathsizes[$i]} => $mpathsize"
+      let i=$i+1
+    done
   fi
 }
 
@@ -922,47 +1009,52 @@ if test @$1 = @--help -o @$1 = @-h -o @$1 = @-?; then
     echo "Usage: rescan-scsi-bus.sh [options] [host [host ...]]"
     echo "Options:"
     echo " -a      scan all targets, not just currently existing [default: disabled]"
+    echo " -c      enables scanning of channels 0 1   [default: 0 / all detected ones]"
     echo " -d      enable debug                       [default: 0]"
+    echo " -f      flush failed multipath devices     [default: disabled]"
+    echo " -h      help: print this usage message then exit"
+    echo " -i      issue a FibreChannel LIP reset     [default: disabled]"
+    echo " -I SECS issue a FibreChannel LIP reset and wait for SECS seconds [default: disabled]"
     echo " -l      activates scanning for LUNs 0--7   [default: 0]"
     echo " -L NUM  activates scanning for LUNs 0--NUM [default: 0]"
-    echo " -w      scan for target device IDs 0--15   [default: 0--7]"
-    echo " -c      enables scanning of channels 0 1   [default: 0 / all detected ones]"
     echo " -r      enables removing of devices        [default: disabled]"
-    echo " -f      flush failed multipath devices     [default: disabled]"
-    echo " -i      issue a FibreChannel LIP reset     [default: disabled]"
-    echo " -u      look for existing disks that have been remapped"
     echo " -s      look for resized disks and reload associated multipath devices, if applicable"
+    echo " -u      look for existing disks that have been remapped"
+    echo " -w      scan for target device IDs 0--15   [default: 0--7]"
     echo "--alltargets:    same as -a"
-    echo "--remove:        same as -r"
-    echo "--flush:         same as -f"
-    echo "--issue-lip:     same as -i"
-    echo "--wide:          same as -w"
-    echo "--forcerescan:   Rescan existing devices"
-    echo "--forceremove:   Remove and readd every device (DANGEROUS)"
-    echo "--nooptscan:     don't stop looking for LUNs is 0 is not found"
-    echo "--color:         use coloured prefixes OLD/NEW/DEL"
-    echo "--hosts=LIST:    Scan only host(s) in LIST"
-    echo "--channels=LIST: Scan only channel(s) in LIST"
-    echo "--ids=LIST:      Scan only target ID(s) in LIST"
-    echo "--luns=LIST:     Scan only lun(s) in LIST"  
-    echo "--sync/nosync:   Issue a sync / no sync [default: sync if remove]"
     echo "--attachpq3:     Tell kernel to attach sg to LUN 0 that reports PQ=3"
-    echo "--reportlun2:    Tell kernel to try REPORT_LUN even on SCSI2 devices"
+    echo "--channels=LIST: Scan only channel(s) in LIST"
+    echo "--color:         use coloured prefixes OLD/NEW/DEL"
+    echo "--flush:         same as -f"
+    echo "--forceremove:   Remove and readd every device (DANGEROUS)"
+    echo "--forcerescan:   Rescan existing devices"
+    echo "--help:          print this usage message then exit"
+    echo "--hosts=LIST:    Scan only host(s) in LIST"
+    echo "--ids=LIST:      Scan only target ID(s) in LIST"
+    echo "--issue-lip:     same as -i"
+    echo "--issue-lip-wait=SECS:     same as -I"
     echo "--largelun:      Tell kernel to support LUNs > 7 even on SCSI2 devs"
-    echo "--sparselun:     Tell kernel to support sparse LUN numbering"
-    echo "--update:        same as -u"
+    echo "--luns=LIST:     Scan only lun(s) in LIST"  
+    echo "--nooptscan:     don't stop looking for LUNs is 0 is not found"
+    echo "--remove:        same as -r"
+    echo "--reportlun2:    Tell kernel to try REPORT_LUN even on SCSI2 devices"
     echo "--resize:        same as -s"
-    echo " Host numbers may thus be specified either directly on cmd line (deprecated) or"
-    echo " or with the --hosts=LIST parameter (recommended)."
+    echo "--sparselun:     Tell kernel to support sparse LUN numbering"
+    echo "--sync/nosync:   Issue a sync / no sync [default: sync if remove]"
+    echo "--update:        same as -u"
+    echo "--wide:          same as -w"
+    echo ""
+    echo "Host numbers may thus be specified either directly on cmd line (deprecated)"
+    echo "or with the --hosts=LIST parameter (recommended)."
     echo "LIST: A[-B][,C[-D]]... is a comma separated list of single values and ranges"
-    echo " (No spaces allowed.)"
+    echo "(No spaces allowed.)"
     exit 0
 fi
 
 if test ! -d /sys/class/scsi_host/ -a ! -d /proc/scsi/; then
   echo "Error: SCSI subsystem not active"
   exit 1
-fi	
+fi
 
 # Make sure sg is there
 modprobe sg >/dev/null 2>&1
@@ -1002,6 +1094,7 @@ optscan=1
 sync=1
 existing_targets=1
 mp_enable=
+lipreset=-1
 declare -i scan_flags=0
 
 # Scan options
@@ -1019,7 +1112,8 @@ while test ! -z "$opt" -a -z "${opt##-*}"; do
     c) opt_channelsearch="0 1" ;;
     r) remove=1 ;;
     s) resize=1 ;;
-    i) lipreset=1 ;;
+    i) lipreset=0 ;;
+    I) shift; lipreset=$opt ;;
     u) update=1 ;;
     -alltargets)  existing_targets=;;
     -flush)       flush=1 ;;
@@ -1032,7 +1126,8 @@ while test ! -z "$opt" -a -z "${opt##-*}"; do
     -luns=*)  arg=${opt#-luns=};        lunsearch=`expandlist $arg` ;; 
     -color) setcolor ;;
     -nooptscan) optscan=0 ;;
-    -issue-lip) lipreset=1 ;;
+    -issue-lip) lipreset=0 ;;
+    -issue-lip-wait) lipreset=${opt#-issue-lip-wait=};;
     -sync) sync=2 ;;
     -nosync) sync=0 ;;
     -multipath) mp_enable=1 ;;
@@ -1097,6 +1192,9 @@ fi
 if [ $update -eq 1 ] ; then
   echo "Searching for remapped LUNs"
   findremapped
+  # If you've changed the mapping, there's a chance it's a different size
+  mpaths=""
+  findresized
 # Search for resized LUNs
 elif [ $resize -eq 1 ] ; then
   echo "Searching for resized LUNs"
@@ -1107,9 +1205,11 @@ else
   echo -n "Scanning host $host "
   if test -e /sys/class/fc_host/host$host ; then
     # It's pointless to do a target scan on FC
-    if test -n "$lipreset" ; then
-      echo 1 > /sys/class/fc_host/host$host/issue_lip 2> /dev/null;
+    issue_lip=/sys/class/fc_host/host$host/issue_lip
+    if test -e $issue_lip -a $lipreset -ge 0 ; then
+      echo 1 > $issue_lip 2> /dev/null;
       udevadm_settle
+      [ $lipreset -gt 0 ] && sleep $lipreset
     fi
     channelsearch=
     idsearch=
@@ -1145,13 +1245,12 @@ let rmvd_found=$rmvd+$found
 if test -n "$mp_enable" -a $rmvd_found -gt 0 ; then
   echo "Attempting to update multipath devices..."
   if test $rmvd -gt 0 ; then
-    /sbin/udevadm trigger
     udevadm_settle
     echo "Removing multipath mappings for removed devices if all paths are now failed... "
     flushmpaths 1
   fi
   if test $found -gt 0 ; then
-    /sbin/udevadm trigger
+    /sbin/udevadm trigger --sysname-match=sd*
     udevadm_settle
     echo "Trying to discover new multipath mappings for newly discovered devices... "
     $MULTIPATH | grep "create:" 2> /dev/null
@@ -1162,7 +1261,7 @@ echo "$found new or changed device(s) found.          "
 if test ! -z "$FOUNDDEVS" ; then
   printf "$FOUNDDEVS"
 fi
-echo "$updated remapped or resized device(s) found.		"
+echo "$updated remapped or resized device(s) found."
 if test ! -z "$CHGDEVS" ; then
   printf "$CHGDEVS"
 fi
