@@ -1,7 +1,7 @@
 /* A utility program for copying files. Specialised for "files" that
  * represent devices that understand the SCSI command set.
  *
- * Copyright (C) 1999 - 2013 D. Gilbert and P. Allworth
+ * Copyright (C) 1999 - 2015 D. Gilbert and P. Allworth
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -57,8 +57,10 @@
 #include "sg_cmds_basic.h"
 #include "sg_cmds_extra.h"
 #include "sg_io_linux.h"
+#include "sg_unaligned.h"
+
+static const char * version_str = "5.83 20150215";
 
-static const char * version_str = "5.77 20130507";
 
 #define ME "sg_dd: "
 
@@ -189,13 +191,14 @@ static void
 print_stats(const char * str)
 {
     if (0 != dd_count)
-        fprintf(stderr, "  remaining block count=%"PRId64"\n", dd_count);
-    fprintf(stderr, "%s%"PRId64"+%d records in\n", str, in_full - in_partial,
-            in_partial);
-    fprintf(stderr, "%s%"PRId64"+%d records out\n", str,
+        fprintf(stderr, "  remaining block count=%" PRId64 "\n", dd_count);
+    fprintf(stderr, "%s%" PRId64 "+%d records in\n", str,
+            in_full - in_partial, in_partial);
+    fprintf(stderr, "%s%" PRId64 "+%d records out\n", str,
             out_full - out_partial, out_partial);
     if (oflag.sparse)
-        fprintf(stderr, "%s%"PRId64" bypassed records out\n", str, out_sparse);
+        fprintf(stderr, "%s%" PRId64 " bypassed records out\n", str,
+                out_sparse);
     if (recovered_errs > 0)
         fprintf(stderr, "%s%d recovered errors\n", str, recovered_errs);
     if (num_retries > 0)
@@ -230,7 +233,7 @@ interrupt_handler(int sig)
 static void
 siginfo_handler(int sig)
 {
-    sig = sig;  /* dummy to stop -W warning messages */
+    if (sig) { ; }      /* unused, dummy to suppress warning */
     fprintf(stderr, "Progress report, continuing ...\n");
     if (do_time)
         calc_duration_throughput(1);
@@ -257,12 +260,12 @@ find_bsg_major(void)
         return;
     }
     while ((cp = fgets(b, sizeof(b), fp))) {
-        if ((1 == sscanf(b, "%s", a)) &&
+        if ((1 == sscanf(b, "%126s", a)) &&
             (0 == memcmp(a, "Character", 9)))
             break;
     }
     while (cp && (cp = fgets(b, sizeof(b), fp))) {
-        if (2 == sscanf(b, "%d %s", &n, a)) {
+        if (2 == sscanf(b, "%d %126s", &n, a)) {
             if (0 == strcmp("bsg", a)) {
                 bsg_major = n;
                 break;
@@ -414,7 +417,7 @@ usage()
 static int
 scsi_read_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
 {
-    int k, res;
+    int res;
     unsigned int ui;
     unsigned char rcBuff[RCAP16_REPLY_LEN];
     int verb;
@@ -432,24 +435,18 @@ scsi_read_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
                                verb);
         if (0 != res)
             return res;
-        for (k = 0, ls = 0; k < 8; ++k) {
-            ls <<= 8;
-            ls |= rcBuff[k];
-        }
+        ls = (int64_t)sg_get_unaligned_be64(rcBuff);
         *num_sect = ls + 1;
-        *sect_sz = (rcBuff[8] << 24) | (rcBuff[9] << 16) |
-                   (rcBuff[10] << 8) | rcBuff[11];
+        *sect_sz = (int)sg_get_unaligned_be32(rcBuff + 8);
     } else {
-        ui = ((rcBuff[0] << 24) | (rcBuff[1] << 16) | (rcBuff[2] << 8) |
-              rcBuff[3]);
+        ui = sg_get_unaligned_be32(rcBuff);
         /* take care not to sign extend values > 0x7fffffff */
         *num_sect = (int64_t)ui + 1;
-        *sect_sz = (rcBuff[4] << 24) | (rcBuff[5] << 16) |
-                   (rcBuff[6] << 8) | rcBuff[7];
+        *sect_sz = (int)sg_get_unaligned_be32(rcBuff + 4);
     }
     if (verbose)
-        fprintf(stderr, "      number of blocks=%"PRId64" [0x%"PRIx64"], "
-                "block size=%d\n", *num_sect, *num_sect, *sect_sz);
+        fprintf(stderr, "      number of blocks=%" PRId64 " [0x%" PRIx64
+                "], " "block size=%d\n", *num_sect, *num_sect, *sect_sz);
     return 0;
 }
 
@@ -475,7 +472,7 @@ read_blkdev_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
         *num_sect = ((int64_t)ull / (int64_t)*sect_sz);
         if (verbose)
             fprintf(stderr, "      [bgs64] number of blocks=%" PRId64 " [0x%"
-                    PRIx64"], block size=%d\n", *num_sect, *num_sect,
+                    PRIx64 "], block size=%d\n", *num_sect, *num_sect,
                     *sect_sz);
  #else
         unsigned long ul;
@@ -486,7 +483,7 @@ read_blkdev_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
         }
         *num_sect = (int64_t)ul;
         if (verbose)
-            fprintf(stderr, "      [bgs] number of blocks=%"PRId64" [0x%"
+            fprintf(stderr, "      [bgs] number of blocks=%" PRId64 " [0x%"
                     PRIx64 "],  block size=%d\n", *num_sect, *num_sect,
                     *sect_sz);
  #endif
@@ -520,9 +517,7 @@ sg_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
         sz_ind = 0;
         cdbp[0] = (unsigned char)(write_true ? wr_opcode[sz_ind] :
                                                rd_opcode[sz_ind]);
-        cdbp[1] = (unsigned char)((start_block >> 16) & 0x1f);
-        cdbp[2] = (unsigned char)((start_block >> 8) & 0xff);
-        cdbp[3] = (unsigned char)(start_block & 0xff);
+        sg_put_unaligned_be24(start_block, cdbp + 1);
         cdbp[4] = (256 == blocks) ? 0 : (unsigned char)blocks;
         if (blocks > 256) {
             fprintf(stderr, ME "for 6 byte commands, maximum number of "
@@ -544,12 +539,8 @@ sg_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
         sz_ind = 1;
         cdbp[0] = (unsigned char)(write_true ? wr_opcode[sz_ind] :
                                                rd_opcode[sz_ind]);
-        cdbp[2] = (unsigned char)((start_block >> 24) & 0xff);
-        cdbp[3] = (unsigned char)((start_block >> 16) & 0xff);
-        cdbp[4] = (unsigned char)((start_block >> 8) & 0xff);
-        cdbp[5] = (unsigned char)(start_block & 0xff);
-        cdbp[7] = (unsigned char)((blocks >> 8) & 0xff);
-        cdbp[8] = (unsigned char)(blocks & 0xff);
+        sg_put_unaligned_be32(start_block, cdbp + 2);
+        sg_put_unaligned_be16(blocks, cdbp + 7);
         if (blocks & (~0xffff)) {
             fprintf(stderr, ME "for 10 byte commands, maximum number of "
                             "blocks is %d\n", 0xffff);
@@ -560,31 +551,15 @@ sg_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
         sz_ind = 2;
         cdbp[0] = (unsigned char)(write_true ? wr_opcode[sz_ind] :
                                                rd_opcode[sz_ind]);
-        cdbp[2] = (unsigned char)((start_block >> 24) & 0xff);
-        cdbp[3] = (unsigned char)((start_block >> 16) & 0xff);
-        cdbp[4] = (unsigned char)((start_block >> 8) & 0xff);
-        cdbp[5] = (unsigned char)(start_block & 0xff);
-        cdbp[6] = (unsigned char)((blocks >> 24) & 0xff);
-        cdbp[7] = (unsigned char)((blocks >> 16) & 0xff);
-        cdbp[8] = (unsigned char)((blocks >> 8) & 0xff);
-        cdbp[9] = (unsigned char)(blocks & 0xff);
+        sg_put_unaligned_be32(start_block, cdbp + 2);
+        sg_put_unaligned_be32(blocks, cdbp + 6);
         break;
     case 16:
         sz_ind = 3;
         cdbp[0] = (unsigned char)(write_true ? wr_opcode[sz_ind] :
                                                rd_opcode[sz_ind]);
-        cdbp[2] = (unsigned char)((start_block >> 56) & 0xff);
-        cdbp[3] = (unsigned char)((start_block >> 48) & 0xff);
-        cdbp[4] = (unsigned char)((start_block >> 40) & 0xff);
-        cdbp[5] = (unsigned char)((start_block >> 32) & 0xff);
-        cdbp[6] = (unsigned char)((start_block >> 24) & 0xff);
-        cdbp[7] = (unsigned char)((start_block >> 16) & 0xff);
-        cdbp[8] = (unsigned char)((start_block >> 8) & 0xff);
-        cdbp[9] = (unsigned char)(start_block & 0xff);
-        cdbp[10] = (unsigned char)((blocks >> 24) & 0xff);
-        cdbp[11] = (unsigned char)((blocks >> 16) & 0xff);
-        cdbp[12] = (unsigned char)((blocks >> 8) & 0xff);
-        cdbp[13] = (unsigned char)(blocks & 0xff);
+        sg_put_unaligned_be64(start_block, cdbp + 2);
+        sg_put_unaligned_be32(blocks, cdbp + 10);
         break;
     default:
         fprintf(stderr, ME "expected cdb size of 6, 10, 12, or 16 but got"
@@ -640,7 +615,8 @@ sg_read_low(int sg_fd, unsigned char * buff, int blocks, int64_t from_block,
             fprintf(stderr, "%02x ", rdCmd[k]);
         fprintf(stderr, "\n");
     }
-    while (((res = ioctl(sg_fd, SG_IO, &io_hdr)) < 0) && (EINTR == errno))
+    while (((res = ioctl(sg_fd, SG_IO, &io_hdr)) < 0) &&
+           ((EINTR == errno) || (EAGAIN == errno)))
         ;
     if (res < 0) {
         if (ENOMEM == errno)
@@ -661,12 +637,12 @@ sg_read_low(int sg_fd, unsigned char * buff, int blocks, int64_t from_block,
         info_valid = sg_get_sense_info_fld(sbp, slen, io_addrp);
         if (info_valid) {
             fprintf(stderr, "    lba of last recovered error in this "
-                    "READ=0x%"PRIx64"\n", *io_addrp);
+                    "READ=0x%" PRIx64 "\n", *io_addrp);
             if (verbose > 1)
                 sg_chk_n_print3("reading", &io_hdr, 1);
         } else {
             fprintf(stderr, "Recovered error: [no info] reading from "
-                    "block=0x%"PRIx64", num=%d\n", from_block, blocks);
+                    "block=0x%" PRIx64 ", num=%d\n", from_block, blocks);
             sg_chk_n_print3("reading", &io_hdr, verbose > 1);
         }
         break;
@@ -786,8 +762,8 @@ sg_read(int sg_fd, unsigned char * buff, int blocks, int64_t from_block,
             break;
         case SG_LIB_CAT_MEDIUM_HARD_WITH_INFO:
             if (retries_tmp > 0) {
-                fprintf(stderr, ">>> retrying a sgio read, lba=0x%"PRIx64"\n",
-                        (uint64_t)lba);
+                fprintf(stderr, ">>> retrying a sgio read, lba=0x%" PRIx64
+                        "\n", (uint64_t)lba);
                 --retries_tmp;
                 ++num_retries;
                 if (unrecovered_errs > 0)
@@ -807,8 +783,8 @@ sg_read(int sg_fd, unsigned char * buff, int blocks, int64_t from_block,
             may_coe = 1;
         default:
             if (retries_tmp > 0) {
-                fprintf(stderr, ">>> retrying a sgio read, lba=0x%"PRIx64"\n",
-                        (uint64_t)lba);
+                fprintf(stderr, ">>> retrying a sgio read, lba=0x%" PRIx64
+                        "\n", (uint64_t)lba);
                 --retries_tmp;
                 ++num_retries;
                 if (unrecovered_errs > 0)
@@ -823,10 +799,10 @@ sg_read(int sg_fd, unsigned char * buff, int blocks, int64_t from_block,
             continue;
         if ((io_addr < (uint64_t)lba) ||
             (io_addr >= (uint64_t)(lba + blks))) {
-                fprintf(stderr, "  Unrecovered error lba 0x%"PRIx64" not in "
-                    "correct range:\n\t[0x%"PRIx64",0x%"PRIx64"]\n", io_addr,
-                    (uint64_t)lba,
-                    (uint64_t)(lba + blks - 1));
+                fprintf(stderr, "  Unrecovered error lba 0x%" PRIx64 " not "
+                        "in correct range:\n\t[0x%" PRIx64 ",0x%" PRIx64
+                        "]\n", io_addr, (uint64_t)lba,
+                        (uint64_t)(lba + blks - 1));
             may_coe = 1;
             goto err_out;
         }
@@ -881,7 +857,7 @@ sg_read(int sg_fd, unsigned char * buff, int blocks, int64_t from_block,
         bp += (blks * bs);
         lba += blks;
         if ((0 != ifp->pdt) || (ifp->coe < 2)) {
-            fprintf(stderr, ">> unrecovered read error at blk=%"PRId64", "
+            fprintf(stderr, ">> unrecovered read error at blk=%" PRId64 ", "
                     "pdt=%d, use zeros\n", lba, ifp->pdt);
             memset(bp, 0, bs);
         } else if (io_addr < UINT_MAX) {
@@ -950,8 +926,8 @@ sg_read(int sg_fd, unsigned char * buff, int blocks, int64_t from_block,
                 memset(bp, 0, bs);
             free(buffp);
         } else {
-            fprintf(stderr, ">> read_long(10) cannot handle blk=%"PRId64", "
-                    "use zeros\n", lba);
+            fprintf(stderr, ">> read_long(10) cannot handle blk=%" PRId64
+                    ", use zeros\n", lba);
             memset(bp, 0, bs);
         }
         ++xferred;
@@ -971,7 +947,7 @@ sg_read(int sg_fd, unsigned char * buff, int blocks, int64_t from_block,
 err_out:
     if (ifp->coe) {
         memset(bp, 0, bs * blks);
-        fprintf(stderr, ">> unable to read at blk=%"PRId64" for "
+        fprintf(stderr, ">> unable to read at blk=%" PRId64 " for "
                 "%d bytes, use zeros\n", lba, bs * blks);
         if (blks > 1)
             fprintf(stderr, ">>   try reducing bpt to limit number "
@@ -1005,8 +981,8 @@ sg_write(int sg_fd, unsigned char * buff, int blocks, int64_t to_block,
 
     if (sg_build_scsi_cdb(wrCmd, ofp->cdbsz, blocks, to_block, 1, ofp->fua,
                           ofp->dpo)) {
-        fprintf(stderr, ME "bad wr cdb build, to_block=%"PRId64", blocks=%d\n",
-                to_block, blocks);
+        fprintf(stderr, ME "bad wr cdb build, to_block=%" PRId64
+                ", blocks=%d\n", to_block, blocks);
         return SG_LIB_SYNTAX_ERROR;
     }
 
@@ -1030,7 +1006,8 @@ sg_write(int sg_fd, unsigned char * buff, int blocks, int64_t to_block,
             fprintf(stderr, "%02x ", wrCmd[k]);
         fprintf(stderr, "\n");
     }
-    while (((res = ioctl(sg_fd, SG_IO, &io_hdr)) < 0) && (EINTR == errno))
+    while (((res = ioctl(sg_fd, SG_IO, &io_hdr)) < 0) &&
+           ((EINTR == errno) || (EAGAIN == errno)))
         ;
     if (res < 0) {
         if (ENOMEM == errno)
@@ -1051,12 +1028,12 @@ sg_write(int sg_fd, unsigned char * buff, int blocks, int64_t to_block,
                                            &io_addr);
         if (info_valid) {
             fprintf(stderr, "    lba of last recovered error in this "
-                    "WRITE=0x%"PRIx64"\n", io_addr);
+                    "WRITE=0x%" PRIx64 "\n", io_addr);
             if (verbose > 1)
                 sg_chk_n_print3("writing", &io_hdr, 1);
         } else {
             fprintf(stderr, "Recovered error: [no info] writing to "
-                    "block=0x%"PRIx64", num=%d\n", to_block, blocks);
+                    "block=0x%" PRIx64 ", num=%d\n", to_block, blocks);
             sg_chk_n_print3("writing", &io_hdr, verbose > 1);
         }
         break;
@@ -1073,7 +1050,7 @@ sg_write(int sg_fd, unsigned char * buff, int blocks, int64_t to_block,
         sg_chk_n_print3("writing", &io_hdr, verbose > 1);
         ++unrecovered_errs;
         if (ofp->coe) {
-            fprintf(stderr, ">> ignored errors for out blk=%"PRId64" for "
+            fprintf(stderr, ">> ignored errors for out blk=%" PRId64 " for "
                     "%d bytes\n", to_block, bs * blocks);
             return 0; /* fudge success */
         } else
@@ -1323,7 +1300,7 @@ open_if(const char * inf, int64_t skip, int bpt, struct flags_t * ifp,
                 }
                 if (verbose)
                     fprintf(stderr, "  >> skip: lseek64 SEEK_SET, "
-                            "byte offset=0x%"PRIx64"\n",
+                            "byte offset=0x%" PRIx64 "\n",
                             (uint64_t)offset);
             }
 #ifdef HAVE_POSIX_FADVISE
@@ -1465,7 +1442,7 @@ open_of(const char * outf, int64_t seek, int bpt, struct flags_t * ofp,
             }
             if (verbose)
                 fprintf(stderr, "   >> seek: lseek64 SEEK_SET, "
-                        "byte offset=0x%"PRIx64"\n",
+                        "byte offset=0x%" PRIx64 "\n",
                         (uint64_t)offset);
         }
     }
@@ -1832,7 +1809,7 @@ main(int argc, char * argv[])
             out_num_sect -= seek;
 #ifdef SG_DEBUG
         fprintf(stderr, "Start of loop, count=%" PRId64 ", in_num_sect=%"
-                PRId64 ", out_num_sect=%"PRId64"\n", dd_count, in_num_sect,
+                PRId64 ", out_num_sect=%" PRId64 "\n", dd_count, in_num_sect,
                 out_num_sect);
 #endif
         if (dd_count < 0) {
@@ -1868,15 +1845,35 @@ main(int argc, char * argv[])
 
     if (iflag.dio || iflag.direct || oflag.direct || (FT_RAW & in_type) ||
         (FT_RAW & out_type)) {
-        size_t psz = sysconf(_SC_PAGESIZE); /* was getpagesize() */
+        size_t psz;
 
+#if defined(HAVE_SYSCONF) && defined(_SC_PAGESIZE)
+        psz = sysconf(_SC_PAGESIZE); /* POSIX.1 (was getpagesize()) */
+#else
+        psz = 4096;     /* give up, pick likely figure */
+#endif
+
+#ifdef HAVE_POSIX_MEMALIGN
+        {
+            int err;
+
+            err = posix_memalign((void **)&wrkBuff, psz, blk_sz * bpt);
+            if (err) {
+                fprintf(stderr, "posix_memalign: error [%d] out of memory?\n",
+                        err);
+                return SG_LIB_CAT_OTHER;
+            }
+            wrkPos = wrkBuff;
+        }
+#else
         wrkBuff = (unsigned char*)malloc(blk_sz * bpt + psz);
         if (0 == wrkBuff) {
-            fprintf(stderr, "Not enough user memory for raw\n");
+            fprintf(stderr, "Not enough user memory for work buffer\n");
             return SG_LIB_CAT_OTHER;
         }
-        wrkPos = (unsigned char *)(((unsigned long)wrkBuff + psz - 1) &
+        wrkPos = (unsigned char *)(((uintptr_t)wrkBuff + psz - 1) &
                                    (~(psz - 1)));
+#endif
     } else {
         wrkBuff = (unsigned char*)malloc(blk_sz * bpt);
         if (0 == wrkBuff) {
@@ -1888,7 +1885,7 @@ main(int argc, char * argv[])
 
     blocks_per = bpt;
 #ifdef SG_DEBUG
-    fprintf(stderr, "Start of loop, count=%"PRId64", blocks_per=%d\n",
+    fprintf(stderr, "Start of loop, count=%" PRId64 ", blocks_per=%d\n",
             dd_count, blocks_per);
 #endif
     if (do_time) {
@@ -1930,8 +1927,8 @@ main(int argc, char * argv[])
                 }
             }
             if (res) {
-                fprintf(stderr, "sg_read failed,%s at or after lba=%"PRId64" "
-                        "[0x%"PRIx64"]\n",
+                fprintf(stderr, "sg_read failed,%s at or after lba=%" PRId64
+                        " [0x%" PRIx64 "]\n",
                         ((-2 == res) ? " try reducing bpt," : ""), skip, skip);
                 ret = res;
                 break;
@@ -1946,13 +1943,14 @@ main(int argc, char * argv[])
             }
         } else {
             while (((res = read(infd, wrkPos, blocks * blk_sz)) < 0) &&
-                   (EINTR == errno))
+                   ((EINTR == errno) || (EAGAIN == errno)))
                 ;
             if (verbose > 2)
                 fprintf(stderr, "read(unix): count=%d, res=%d\n",
                         blocks * blk_sz, res);
             if (res < 0) {
-                snprintf(ebuff, EBUFF_SZ, ME "reading, skip=%"PRId64" ", skip);
+                snprintf(ebuff, EBUFF_SZ, ME "reading, skip=%" PRId64 " ",
+                         skip);
                 perror(ebuff);
                 ret = -1;
                 break;
@@ -1972,15 +1970,15 @@ main(int argc, char * argv[])
             break;      /* nothing read so leave loop */
 
         if (out2f[0]) {
-            while (((res = write(out2fd, wrkPos, blocks * blk_sz)) < 0)
-                   && (EINTR == errno))
+            while (((res = write(out2fd, wrkPos, blocks * blk_sz)) < 0) &&
+                   ((EINTR == errno) || (EAGAIN == errno)))
                 ;
             if (verbose > 2)
                 fprintf(stderr, "write to of2: count=%d, res=%d\n",
                         blocks * blk_sz, res);
             if (res < 0) {
-                snprintf(ebuff, EBUFF_SZ, ME "writing to of2, seek=%"PRId64" ",
-                         seek);
+                snprintf(ebuff, EBUFF_SZ, ME "writing to of2, seek=%" PRId64
+                         " ", seek);
                 perror(ebuff);
                 ret = -1;
                 break;
@@ -2008,7 +2006,8 @@ main(int argc, char * argv[])
                 out_sparse += blocks;
                 if (verbose > 2)
                     fprintf(stderr, "sparse bypassing sg_write: seek "
-                            "blk=%"PRId64", offset blks=%d\n", seek, blocks);
+                            "blk=%" PRId64 ", offset blks=%d\n", seek,
+                            blocks);
             } else if (FT_DEV_NULL & out_type)
                 ;
             else {
@@ -2077,7 +2076,7 @@ main(int argc, char * argv[])
                     break;
                 else if (retries_tmp > 0) {
                     fprintf(stderr, ">>> retrying a sgio write, "
-                            "lba=0x%"PRIx64"\n", (uint64_t)seek);
+                            "lba=0x%" PRIx64 "\n", (uint64_t)seek);
                     --retries_tmp;
                     ++num_retries;
                     if (unrecovered_errs > 0)
@@ -2087,7 +2086,7 @@ main(int argc, char * argv[])
                 first = 0;
             }
             if (0 != ret) {
-                fprintf(stderr, "sg_write failed,%s seek=%"PRId64"\n",
+                fprintf(stderr, "sg_write failed,%s seek=%" PRId64 "\n",
                         ((-2 == ret) ? " try reducing bpt," : ""), seek);
                 break;
             } else {
@@ -2098,14 +2097,15 @@ main(int argc, char * argv[])
         } else if (FT_DEV_NULL & out_type)
             out_full += blocks; /* act as if written out without error */
         else {
-            while (((res = write(outfd, wrkPos, blocks * blk_sz)) < 0)
-                   && (EINTR == errno))
+            while (((res = write(outfd, wrkPos, blocks * blk_sz)) < 0) &&
+                   ((EINTR == errno) || (EAGAIN == errno)))
                 ;
             if (verbose > 2)
                 fprintf(stderr, "write(unix): count=%d, res=%d\n",
                         blocks * blk_sz, res);
             if (res < 0) {
-                snprintf(ebuff, EBUFF_SZ, ME "writing, seek=%"PRId64" ", seek);
+                snprintf(ebuff, EBUFF_SZ, ME "writing, seek=%" PRId64 " ",
+                         seek);
                 perror(ebuff);
                 ret = -1;
                 break;
@@ -2138,19 +2138,19 @@ main(int argc, char * argv[])
                 // rt = posix_fadvise(infd, 0, 0, POSIX_FADV_DONTNEED);
                 if (rt)         /* returns error as result */
                     fprintf(stderr, "posix_fadvise on read, skip="
-                            "%"PRId64" ,err=%d\n", skip, rt);
+                            "%" PRId64 " ,err=%d\n", skip, rt);
             }
             if ((oflag.nocache & 2) && (bytes_of2 > 0) && out2_valid) {
                 rt = posix_fadvise(out2fd, 0, 0, POSIX_FADV_DONTNEED);
                 if (rt)
                     fprintf(stderr, "posix_fadvise on of2, seek="
-                            "%"PRId64" ,err=%d\n", seek, rt);
+                            "%" PRId64 " ,err=%d\n", seek, rt);
             }
             if ((oflag.nocache & 1) && (bytes_of > 0) && out_valid) {
                 rt = posix_fadvise(outfd, 0, 0, POSIX_FADV_DONTNEED);
                 if (rt)
                     fprintf(stderr, "posix_fadvise on output, seek="
-                            "%"PRId64" ,err=%d\n", seek, rt);
+                            "%" PRId64 " ,err=%d\n", seek, rt);
             }
         }
 #endif
@@ -2166,14 +2166,14 @@ main(int argc, char * argv[])
         else {
             /* ... try writing to extend ofile to length prior to error */
             while (((res = write(outfd, zeros_buff, penult_blocks * blk_sz))
-                    < 0) && (EINTR == errno))
+                    < 0) && ((EINTR == errno) || (EAGAIN == errno)))
                 ;
             if (verbose > 2)
                 fprintf(stderr, "write(unix, sparse after error): count=%d, "
                         "res=%d\n", penult_blocks * blk_sz, res);
             if (res < 0) {
                 snprintf(ebuff, EBUFF_SZ, ME "writing(sparse after error), "
-                        "seek=%"PRId64" ", seek);
+                        "seek=%" PRId64 " ", seek);
                 perror(ebuff);
             }
         }
