@@ -1,7 +1,7 @@
 /* This code is does a SCSI READ CAPACITY command on the given device
    and outputs the result.
 
-*  Copyright (C) 1999 - 2013 D. Gilbert
+*  Copyright (C) 1999 - 2014 D. Gilbert
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
@@ -27,9 +27,10 @@
 #endif
 #include "sg_lib.h"
 #include "sg_cmds_basic.h"
+#include "sg_unaligned.h"
 
 
-static const char * version_str = "3.88 20130507";
+static const char * version_str = "3.93 20141220";
 
 #define ME "sg_readcap: "
 
@@ -37,19 +38,21 @@ static const char * version_str = "3.88 20130507";
 #define RCAP16_REPLY_LEN 32
 
 static struct option long_options[] = {
-        {"brief", 0, 0, 'b'},
-        {"help", 0, 0, 'h'},
-        {"hex", 0, 0, 'H'},
-        {"lba", 1, 0, 'L'},
-        {"long", 0, 0, 'l'},
-        {"16", 0, 0, 'l'},
-        {"new", 0, 0, 'N'},
-        {"old", 0, 0, 'O'},
-        {"pmi", 0, 0, 'p'},
-        {"raw", 0, 0, 'r'},
-        {"verbose", 0, 0, 'v'},
-        {"version", 0, 0, 'V'},
-        {0, 0, 0, 0},
+    {"brief", 0, 0, 'b'},
+    {"help", 0, 0, 'h'},
+    {"hex", 0, 0, 'H'},
+    {"lba", 1, 0, 'L'},
+    {"long", 0, 0, 'l'},
+    {"16", 0, 0, 'l'},
+    {"new", 0, 0, 'N'},
+    {"old", 0, 0, 'O'},
+    {"pmi", 0, 0, 'p'},
+    {"raw", 0, 0, 'r'},
+    {"readonly", 0, 0, 'R'},
+    {"verbose", 0, 0, 'v'},
+    {"version", 0, 0, 'V'},
+    {"zbc", 0, 0, 'z'},
+    {0, 0, 0, 0},
 };
 
 struct opts_t {
@@ -60,19 +63,25 @@ struct opts_t {
     int do_long;
     int do_pmi;
     int do_raw;
+    int o_readonly;
     int do_verbose;
     int do_version;
+    int do_zbc;
     uint64_t llba;
     const char * device_name;
     int opt_new;
 };
 
-static void usage()
+
+static void
+usage()
 {
-    fprintf(stderr, "Usage: sg_readcap [--brief] [--help] [--hex] "
-            "[--lba=LBA] [--long] [--16]\n"
-            "                  [--pmi] [--raw] [--verbose] [--version] "
-            "DEVICE\n"
+    fprintf(stderr,
+            "Usage: sg_readcap [--brief] [--help] [--hex] [--lba=LBA] "
+            "[--long] [--16]\n"
+            "                  [--pmi] [--raw] [--readonly] [--verbose] "
+            "[--version]\n"
+            "                  [--zbc] DEVICE\n"
             "  where:\n"
             "    --brief|-b      brief, two hex numbers: number of blocks "
             "and block size\n"
@@ -91,16 +100,20 @@ static void usage()
             "                    total disk capacity) [made obsolete in "
             "sbc3r26]\n"
             "    --raw|-r        output response in binary to stdout\n"
+            "    --readonly|-R    open DEVICE read-only (def: RCAP(16) "
+            "read-write)\n"
             "    --verbose|-v    increase verbosity\n"
-            "    --version|-V    print version string and exit\n\n"
+            "    --version|-V    print version string and exit\n"
+            "    --zbc|-z        show rc_basis ZBC field (implies --16)\n\n"
             "Perform a SCSI READ CAPACITY (10 or 16) command\n");
 }
 
-static void usage_old()
+static void
+usage_old()
 {
     fprintf(stderr, "Usage:  sg_readcap [-16] [-b] [-h] [-H] [-lba=LBA] "
-            "[-pmi] [-r] [-v] [-V]\n"
-            "                   DEVICE\n"
+            "[-pmi] [-r] [-R]\n"
+            "                   [-v] [-V] [-z] DEVICE\n"
             "  where:\n"
             "    -16    use READ CAPACITY (16) cdb (def: use "
             "10 byte cdb)\n"
@@ -116,20 +129,24 @@ static void usage_old()
             "shows total\n"
             "           disk capacity)\n"
             "    -r     output response in binary to stdout\n"
+            "    -R     open DEVICE read-only (def: RCAP(16) read-write)\n"
             "    -v     increase verbosity\n"
-            "    -V     print version string and exit\n\n"
-            "Perform a SCSI READ CAPACITY command\n");
+            "    -V     print version string and exit\n"
+            "    -z     show rc_basis ZBC field (implies -16)\n\n"
+            "Perform a SCSI READ CAPACITY (10 or 16) command\n");
 }
 
-static void usage_for(const struct opts_t * optsp)
+static void
+usage_for(const struct opts_t * op)
 {
-    if (optsp->opt_new)
+    if (op->opt_new)
         usage();
     else
         usage_old();
 }
 
-static int process_cl_new(struct opts_t * optsp, int argc, char * argv[])
+static int
+process_cl_new(struct opts_t * op, int argc, char * argv[])
 {
     int c;
     int a_one = 0;
@@ -138,7 +155,7 @@ static int process_cl_new(struct opts_t * optsp, int argc, char * argv[])
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "16bhHlL:NOprvV", long_options,
+        c = getopt_long(argc, argv, "16bhHlL:NOprRvVz", long_options,
                         &option_index);
         if (c == -1)
             break;
@@ -149,20 +166,20 @@ static int process_cl_new(struct opts_t * optsp, int argc, char * argv[])
             break;
         case '6':
             if (a_one)
-                ++optsp->do_long;
+                ++op->do_long;
             break;
         case 'b':
-            ++optsp->do_brief;
+            ++op->do_brief;
             break;
         case 'h':
         case '?':
-            ++optsp->do_help;
+            ++op->do_help;
             break;
         case 'H':
-            ++optsp->do_hex;
+            ++op->do_hex;
             break;
         case 'l':
-            ++optsp->do_long;
+            ++op->do_long;
             break;
         case 'L':
             nn = sg_get_llnum(optarg);
@@ -171,40 +188,46 @@ static int process_cl_new(struct opts_t * optsp, int argc, char * argv[])
                 usage();
                 return SG_LIB_SYNTAX_ERROR;
             }
-            optsp->llba = nn;
+            op->llba = nn;
             /* force READ_CAPACITY16 for large lbas */
-            if (optsp->llba > 0xfffffffeULL)
-                ++optsp->do_long;
-            ++optsp->do_lba;
+            if (op->llba > 0xfffffffeULL)
+                ++op->do_long;
+            ++op->do_lba;
             break;
         case 'N':
             break;      /* ignore */
         case 'O':
-            optsp->opt_new = 0;
+            op->opt_new = 0;
             return 0;
         case 'p':
-            ++optsp->do_pmi;
+            ++op->do_pmi;
             break;
         case 'r':
-            ++optsp->do_raw;
+            ++op->do_raw;
+            break;
+        case 'R':
+            ++op->o_readonly;
             break;
         case 'v':
-            ++optsp->do_verbose;
+            ++op->do_verbose;
             break;
         case 'V':
-            ++optsp->do_version;
+            ++op->do_version;
+            break;
+        case 'z':
+            ++op->do_zbc;
             break;
         default:
             fprintf(stderr, "unrecognised option code %c [0x%x]\n", c, c);
-            if (optsp->do_help)
+            if (op->do_help)
                 break;
             usage();
             return SG_LIB_SYNTAX_ERROR;
         }
     }
     if (optind < argc) {
-        if (NULL == optsp->device_name) {
-            optsp->device_name = argv[optind];
+        if (NULL == op->device_name) {
+            op->device_name = argv[optind];
             ++optind;
         }
         if (optind < argc) {
@@ -218,7 +241,8 @@ static int process_cl_new(struct opts_t * optsp, int argc, char * argv[])
     return 0;
 }
 
-static int process_cl_old(struct opts_t * optsp, int argc, char * argv[])
+static int
+process_cl_old(struct opts_t * op, int argc, char * argv[])
 {
     int k, jmp_out, plen, num;
     const char * cp;
@@ -234,43 +258,49 @@ static int process_cl_old(struct opts_t * optsp, int argc, char * argv[])
                 switch (*cp) {
                 case '1':
                     if ('6' == *(cp + 1)) {
-                        ++optsp->do_long;
+                        ++op->do_long;
                         ++cp;
                         --plen;
                     } else
                         jmp_out = 1;
                     break;
                 case 'b':
-                    ++optsp->do_brief;
+                    ++op->do_brief;
                     break;
                 case 'h':
                 case '?':
-                    ++optsp->do_help;
+                    ++op->do_help;
                     break;
                 case 'H':
-                    ++optsp->do_hex;
+                    ++op->do_hex;
                     break;
                 case 'N':
-                    optsp->opt_new = 1;
+                    op->opt_new = 1;
                     return 0;
                 case 'O':
                     break;
                 case 'p':
                     if (0 == strncmp("pmi", cp, 3)) {
-                        ++optsp->do_pmi;
+                        ++op->do_pmi;
                         cp += 2;
                         plen -= 2;
                     } else
                         jmp_out = 1;
                     break;
                 case 'r':
-                    ++optsp->do_raw;
+                    ++op->do_raw;
+                    break;
+                case 'R':
+                    ++op->o_readonly;
                     break;
                 case 'v':
-                    ++optsp->do_verbose;
+                    ++op->do_verbose;
                     break;
                 case 'V':
-                    ++optsp->do_version;
+                    ++op->do_version;
+                    break;
+                case 'z':
+                    ++op->do_zbc;
                     break;
                 default:
                     jmp_out = 1;
@@ -290,9 +320,9 @@ static int process_cl_old(struct opts_t * optsp, int argc, char * argv[])
                 }
                 /* force READ_CAPACITY16 for large lbas */
                 if (uu > 0xfffffffeULL)
-                    ++optsp->do_long;
-                optsp->llba = uu;
-                ++optsp->do_lba;
+                    ++op->do_long;
+                op->llba = uu;
+                ++op->do_lba;
             } else if (0 == strncmp("-old", cp, 4))
                 ;
             else if (jmp_out) {
@@ -300,11 +330,11 @@ static int process_cl_old(struct opts_t * optsp, int argc, char * argv[])
                 usage();
                 return SG_LIB_SYNTAX_ERROR;
             }
-        } else if (0 == optsp->device_name)
-            optsp->device_name = cp;
+        } else if (0 == op->device_name)
+            op->device_name = cp;
         else {
             fprintf(stderr, "too many arguments, got: %s, not expecting: "
-                    "%s\n", optsp->device_name, cp);
+                    "%s\n", op->device_name, cp);
             usage();
             return SG_LIB_SYNTAX_ERROR;
         }
@@ -312,27 +342,29 @@ static int process_cl_old(struct opts_t * optsp, int argc, char * argv[])
     return 0;
 }
 
-static int process_cl(struct opts_t * optsp, int argc, char * argv[])
+static int
+process_cl(struct opts_t * op, int argc, char * argv[])
 {
     int res;
     char * cp;
 
     cp = getenv("SG3_UTILS_OLD_OPTS");
     if (cp) {
-        optsp->opt_new = 0;
-        res = process_cl_old(optsp, argc, argv);
-        if ((0 == res) && optsp->opt_new)
-            res = process_cl_new(optsp, argc, argv);
+        op->opt_new = 0;
+        res = process_cl_old(op, argc, argv);
+        if ((0 == res) && op->opt_new)
+            res = process_cl_new(op, argc, argv);
     } else {
-        optsp->opt_new = 1;
-        res = process_cl_new(optsp, argc, argv);
-        if ((0 == res) && (0 == optsp->opt_new))
-            res = process_cl_old(optsp, argc, argv);
+        op->opt_new = 1;
+        res = process_cl_new(op, argc, argv);
+        if ((0 == res) && (0 == op->opt_new))
+            res = process_cl_old(op, argc, argv);
     }
     return res;
 }
 
-static void dStrRaw(const char* str, int len)
+static void
+dStrRaw(const char* str, int len)
 {
     int k;
 
@@ -340,85 +372,117 @@ static void dStrRaw(const char* str, int len)
         printf("%c", str[k]);
 }
 
-int main(int argc, char * argv[])
+static const char *
+rc_basis_str(int rc_basis, char * b, int blen)
 {
-    int sg_fd, k, res, prot_en, p_type;
+    switch (rc_basis) {
+    case 0:
+        snprintf(b, blen, "last contiguous that's not seq write required");
+        break;
+    case 1:
+        snprintf(b, blen, "last LBA on device");
+        break;
+    default:
+        snprintf(b, blen, "reserved (0x%x)", rc_basis);
+        break;
+    }
+    return b;
+}
+
+
+int
+main(int argc, char * argv[])
+{
+    int sg_fd, res, prot_en, p_type, lbppbe, rw_0_flag;
     uint64_t llast_blk_addr;
     int ret = 0;
-    unsigned int last_blk_addr, block_size;
+    uint32_t last_blk_addr, block_size;
     unsigned char resp_buff[RCAP16_REPLY_LEN];
+    char b[80];
     struct opts_t opts;
+    struct opts_t * op;
 
-    memset(&opts, 0, sizeof(opts));
-    res = process_cl(&opts, argc, argv);
+    op = &opts;
+    memset(op, 0, sizeof(opts));
+    res = process_cl(op, argc, argv);
     if (res)
         return SG_LIB_SYNTAX_ERROR;
-    if (opts.do_help) {
-        usage_for(&opts);
+    if (op->do_help) {
+        usage_for(op);
         return 0;
     }
-    if (opts.do_version) {
+    if (op->do_version) {
         fprintf(stderr, "Version string: %s\n", version_str);
         return 0;
     }
 
-    if (NULL == opts.device_name) {
+    if (NULL == op->device_name) {
         fprintf(stderr, "No DEVICE argument given\n");
-        usage_for(&opts);
+        usage_for(op);
         return SG_LIB_SYNTAX_ERROR;
     }
-    if (opts.do_raw) {
+    if (op->do_raw) {
         if (sg_set_binary_mode(STDOUT_FILENO) < 0) {
             perror("sg_set_binary_mode");
             return SG_LIB_FILE_ERROR;
         }
     }
+    if (op->do_zbc) {
+        if (! op->do_long)
+            ++op->do_long;
+    }
 
     memset(resp_buff, 0, sizeof(resp_buff));
 
-    if ((0 == opts.do_pmi) && (opts.llba > 0)) {
+    if ((0 == op->do_pmi) && (op->llba > 0)) {
         fprintf(stderr, ME "lba can only be non-zero when '--pmi' is set\n");
-        usage_for(&opts);
+        usage_for(op);
         return SG_LIB_SYNTAX_ERROR;
     }
-    if ((sg_fd = sg_cmds_open_device(opts.device_name,
-                 (opts.do_long ? 0 /* rw */ : 1), opts.do_verbose)) < 0) {
-        fprintf(stderr, ME "error opening file: %s: %s\n", opts.device_name,
+    if (op->do_long)
+        rw_0_flag = op->o_readonly;
+    else
+        rw_0_flag = 1;  /* RCAP(10) has opened RO in past, so leave */
+    if ((sg_fd = sg_cmds_open_device(op->device_name, rw_0_flag,
+                                     op->do_verbose)) < 0) {
+        fprintf(stderr, ME "error opening file: %s: %s\n", op->device_name,
                 safe_strerror(-sg_fd));
         return SG_LIB_FILE_ERROR;
     }
 
-    if (! opts.do_long) {
-        res = sg_ll_readcap_10(sg_fd, opts.do_pmi, (unsigned int)opts.llba,
-                               resp_buff, RCAP_REPLY_LEN, 1, opts.do_verbose);
+    if (! op->do_long) {
+        res = sg_ll_readcap_10(sg_fd, op->do_pmi, (unsigned int)op->llba,
+                               resp_buff, RCAP_REPLY_LEN, 1, op->do_verbose);
         ret = res;
         if (0 == res) {
-            if (opts.do_hex || opts.do_raw) {
-                if (opts.do_raw)
+            if (op->do_hex || op->do_raw) {
+                if (op->do_raw)
                     dStrRaw((const char *)resp_buff, RCAP_REPLY_LEN);
+                else if (op->do_hex > 2)
+                    dStrHex((const char *)resp_buff, RCAP_REPLY_LEN, -1);
                 else
                     dStrHex((const char *)resp_buff, RCAP_REPLY_LEN, 1);
                 goto good;
             }
-            last_blk_addr = ((resp_buff[0] << 24) | (resp_buff[1] << 16) |
-                             (resp_buff[2] << 8) | resp_buff[3]);
+            last_blk_addr = sg_get_unaligned_be32(resp_buff + 0);
             if (0xffffffff != last_blk_addr) {
-                block_size = ((resp_buff[4] << 24) | (resp_buff[5] << 16) |
-                             (resp_buff[6] << 8) | resp_buff[7]);
-                if (opts.do_brief) {
-                    printf("0x%x 0x%x\n", last_blk_addr + 1, block_size);
+                block_size = sg_get_unaligned_be32(resp_buff + 4);
+                if (op->do_brief) {
+                    printf("0x%" PRIx32 " 0x%" PRIx32 "\n",
+                           last_blk_addr + 1, block_size);
                     goto good;
                 }
                 printf("Read Capacity results:\n");
-                if (opts.do_pmi)
+                if (op->do_pmi)
                     printf("   PMI mode: given lba=0x%" PRIx64 ", last lba "
-                           "before delay=0x%x\n", opts.llba, last_blk_addr);
+                           "before delay=0x%" PRIx32 "\n", op->llba,
+                           last_blk_addr);
                 else
-                    printf("   Last logical block address=%u (0x%x), Number "
-                           "of blocks=%u\n", last_blk_addr, last_blk_addr,
-                           last_blk_addr + 1);
+                    printf("   Last logical block address=%" PRIu32 " (0x%"
+                           PRIx32 "), Number of blocks=%" PRIu32 "\n",
+                           last_blk_addr, last_blk_addr, last_blk_addr + 1);
                 printf("   Logical block length=%u bytes\n", block_size);
-                if (! opts.do_pmi) {
+                if (! op->do_pmi) {
                     uint64_t total_sz = last_blk_addr + 1;
                     double sz_mb, sz_gb;
 
@@ -440,51 +504,44 @@ int main(int argc, char * argv[])
             } else {
                 printf("READ CAPACITY (10) indicates device capacity too "
                        "large\n  now trying 16 byte cdb variant\n");
-                opts.do_long = 1;
+                op->do_long = 1;
             }
         } else if (SG_LIB_CAT_INVALID_OP == res) {
-            opts.do_long = 1;
+            op->do_long = 1;
             sg_cmds_close_device(sg_fd);
-            if ((sg_fd = sg_cmds_open_device(opts.device_name, 0 /*rw */,
-                                             opts.do_verbose))
-                < 0) {
+            if ((sg_fd = sg_cmds_open_device(op->device_name, op->o_readonly,
+                                             op->do_verbose)) < 0) {
                 fprintf(stderr, ME "error re-opening file: %s (rw): %s\n",
-                        opts.device_name, safe_strerror(-sg_fd));
+                        op->device_name, safe_strerror(-sg_fd));
                 return SG_LIB_FILE_ERROR;
             }
-            if (opts.do_verbose)
+            if (op->do_verbose)
                 fprintf(stderr, "READ CAPACITY (10) not supported, trying "
                         "READ CAPACITY (16)\n");
-        } else if (SG_LIB_CAT_ILLEGAL_REQ == res)
-            fprintf(stderr, "bad field in READ CAPACITY (10) cdb\n");
-        else if (SG_LIB_CAT_NOT_READY == res)
-            fprintf(stderr, "READ CAPACITY (10) failed, device not ready\n");
-        else if (SG_LIB_CAT_ABORTED_COMMAND == res)
-            fprintf(stderr, "READ CAPACITY (10) failed, aborted command\n");
-        else if (! opts.do_verbose)
-            fprintf(stderr, "READ CAPACITY (10) failed [res=%d], try "
-                    "with '-v'\n", res);
+        } else if (res) {
+            sg_get_category_sense_str(res, sizeof(b), b, op->do_verbose);
+            fprintf(stderr, "READ CAPACITY (10) failed: %s\n", b);
+        }
     }
-    if (opts.do_long) {
-        res = sg_ll_readcap_16(sg_fd, opts.do_pmi, opts.llba, resp_buff,
-                               RCAP16_REPLY_LEN, 1, opts.do_verbose);
+    if (op->do_long) {
+        res = sg_ll_readcap_16(sg_fd, op->do_pmi, op->llba, resp_buff,
+                               RCAP16_REPLY_LEN, 1, op->do_verbose);
         ret = res;
         if (0 == res) {
-            if (opts.do_hex || opts.do_raw) {
-                if (opts.do_raw)
+            if (op->do_hex || op->do_raw) {
+                if (op->do_raw)
                     dStrRaw((const char *)resp_buff, RCAP16_REPLY_LEN);
+                else if (op->do_hex > 2)
+                    dStrHex((const char *)resp_buff, RCAP16_REPLY_LEN, -1);
                 else
                     dStrHex((const char *)resp_buff, RCAP16_REPLY_LEN, 1);
                 goto good;
             }
-            for (k = 0, llast_blk_addr = 0; k < 8; ++k) {
-                llast_blk_addr <<= 8;
-                llast_blk_addr |= resp_buff[k];
-            }
-            block_size = ((resp_buff[8] << 24) | (resp_buff[9] << 16) |
-                          (resp_buff[10] << 8) | resp_buff[11]);
-            if (opts.do_brief) {
-                printf("0x%" PRIx64 " 0x%x\n", llast_blk_addr + 1, block_size);
+            llast_blk_addr = sg_get_unaligned_be64(resp_buff + 0);
+            block_size = sg_get_unaligned_be32(resp_buff + 8);
+            if (op->do_brief) {
+                printf("0x%" PRIx64 " 0x%" PRIx32 "\n", llast_blk_addr + 1,
+                       block_size);
                 goto good;
             }
             prot_en = !!(resp_buff[12] & 0x1);
@@ -496,22 +553,34 @@ int main(int argc, char * argv[])
                 printf(" [type %d protection]\n", p_type + 1);
             else
                 printf("\n");
+            if (op->do_zbc) {
+                int rc_basis = (resp_buff[12] >> 4) & 0x3;
+
+                printf("   ZBC's rc_basis=%d [%s]\n", rc_basis,
+                       rc_basis_str(rc_basis, b, sizeof(b)));
+            }
             printf("   Logical block provisioning: lbpme=%d, lbprz=%d\n",
                    !!(resp_buff[14] & 0x80), !!(resp_buff[14] & 0x40));
-            if (opts.do_pmi)
+            if (op->do_pmi)
                 printf("   PMI mode: given lba=0x%" PRIx64 ", last lba "
-                       "before delay=0x%" PRIx64 "\n", opts.llba,
+                       "before delay=0x%" PRIx64 "\n", op->llba,
                        llast_blk_addr);
             else
                 printf("   Last logical block address=%" PRIu64 " (0x%"
                        PRIx64 "), Number of logical blocks=%" PRIu64 "\n",
                        llast_blk_addr, llast_blk_addr, llast_blk_addr + 1);
-            printf("   Logical block length=%u bytes\n", block_size);
-            printf("   Logical blocks per physical block exponent=%d\n",
-                   resp_buff[13] & 0xf);
+            printf("   Logical block length=%" PRIu32 " bytes\n", block_size);
+            lbppbe = resp_buff[13] & 0xf;
+            printf("   Logical blocks per physical block exponent=%d",
+                   lbppbe);
+            if (lbppbe > 0)
+                printf(" [so physical block length=%u bytes]\n",
+                       block_size * (1 << lbppbe));
+            else
+                printf("\n");
             printf("   Lowest aligned logical block address=%d\n",
                    ((resp_buff[14] & 0x3f) << 8) + resp_buff[15]);
-            if (! opts.do_pmi) {
+            if (! op->do_pmi) {
                 uint64_t total_sz = llast_blk_addr + 1;
                 double sz_mb, sz_gb;
 
@@ -530,21 +599,15 @@ int main(int argc, char * argv[])
 #endif
             }
             goto good;
-        }
-        else if (SG_LIB_CAT_INVALID_OP == res)
-            fprintf(stderr, "READ CAPACITY (16) not supported\n");
-        else if (SG_LIB_CAT_NOT_READY == res)
-            fprintf(stderr, "READ CAPACITY (16) failed, device not ready\n");
-        else if (SG_LIB_CAT_ABORTED_COMMAND == res)
-            fprintf(stderr, "READ CAPACITY (16) failed, aborted command\n");
-        else if (SG_LIB_CAT_ILLEGAL_REQ == res)
+        } else if (SG_LIB_CAT_ILLEGAL_REQ == res)
             fprintf(stderr, "bad field in READ CAPACITY (16) cdb "
                     "including unsupported service action\n");
-        else if (! opts.do_verbose)
-            fprintf(stderr, "READ CAPACITY (16) failed [res=%d], try "
-                    "with '-v'\n", res);
+        else if (res) {
+            sg_get_category_sense_str(res, sizeof(b), b, op->do_verbose);
+            fprintf(stderr, "READ CAPACITY (16) failed: %s\n", b);
+        }
     }
-    if (opts.do_brief)
+    if (op->do_brief)
         printf("0x0 0x0\n");
 
 good:
