@@ -1,28 +1,27 @@
 /* A utility program for copying files. Specialised for "files" that
-*  represent devices that understand the SCSI command set.
-*
-*  Copyright (C) 1999 - 2012 D. Gilbert and P. Allworth
-*  This program is free software; you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation; either version 2, or (at your option)
-*  any later version.
-
-   This program is a specialisation of the Unix "dd" command in which
-   one or both of the given files is a scsi generic device or a raw
-   device. A block size ('bs') is assumed to be 512 if not given. This
-   program complains if 'ibs' or 'obs' are given with some other value
-   than 'bs'. If 'if' is not given or 'if=-' then stdin is assumed. If
-   'of' is not given or 'of=-' then stdout assumed.
-
-   A non-standard argument "bpt" (blocks per transfer) is added to control
-   the maximum number of blocks in each transfer. The default value is 128.
-   For example if "bs=512" and "bpt=32" then a maximum of 32 blocks (16 KiB
-   in this case) are transferred to or from the sg device in a single SCSI
-   command.
-
-   This version is designed for the linux kernel 2.4, 2.6 and 3 series.
-
-*/
+ * represent devices that understand the SCSI command set.
+ *
+ * Copyright (C) 1999 - 2015 D. Gilbert and P. Allworth
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is a specialisation of the Unix "dd" command in which
+ * one or both of the given files is a scsi generic device or a raw
+ * device. A block size ('bs') is assumed to be 512 if not given. This
+ * program complains if 'ibs' or 'obs' are given with some other value
+ * than 'bs'. If 'if' is not given or 'if=-' then stdin is assumed. If
+ * 'of' is not given or 'of=-' then stdout assumed.
+ *
+ * A non-standard argument "bpt" (blocks per transfer) is added to control
+ * the maximum number of blocks in each transfer. The default value is 128.
+ * For example if "bs=512" and "bpt=32" then a maximum of 32 blocks (16 KiB
+ * in this case) are transferred to or from the sg device in a single SCSI
+ * command.
+ *
+ * This version is designed for the linux kernel 2.4, 2.6 and 3 series.
+ */
 
 #define _XOPEN_SOURCE 500
 #ifndef _GNU_SOURCE
@@ -57,7 +56,7 @@
 #include "sg_io_linux.h"
 
 
-static char * version_str = "5.43 20121211";
+static const char * version_str = "5.49 20150129";
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_BLOCKS_PER_TRANSFER 128
@@ -69,7 +68,7 @@ static char * version_str = "5.43 20121211";
 
 /* #define SG_DEBUG */
 
-#define SENSE_BUFF_LEN 32       /* Arbitrary, could be larger */
+#define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
 #define READ_CAP_REPLY_LEN 8
 #define RCAP16_REPLY_LEN 32
 
@@ -225,14 +224,14 @@ print_stats(const char * str)
     int64_t infull, outfull;
 
     if (0 != rcoll.out_rem_count)
-        fprintf(stderr, "  remaining block count=%"PRId64"\n",
+        fprintf(stderr, "  remaining block count=%" PRId64 "\n",
                 rcoll.out_rem_count);
     infull = dd_count - rcoll.in_rem_count;
-    fprintf(stderr, "%s%"PRId64"+%d records in\n", str,
+    fprintf(stderr, "%s%" PRId64 "+%d records in\n", str,
             infull - rcoll.in_partial, rcoll.in_partial);
 
     outfull = dd_count - rcoll.out_rem_count;
-    fprintf(stderr, "%s%"PRId64"+%d records out\n", str,
+    fprintf(stderr, "%s%" PRId64 "+%d records out\n", str,
             outfull - rcoll.out_partial, rcoll.out_partial);
 }
 
@@ -255,7 +254,7 @@ interrupt_handler(int sig)
 static void
 siginfo_handler(int sig)
 {
-    sig = sig;  /* dummy to stop -W warning messages */
+    if (sig) { ; }      /* unused, dummy to suppress warning */
     fprintf(stderr, "Progress report, continuing ...\n");
     if (do_time)
         calc_duration_throughput(1);
@@ -293,11 +292,12 @@ tsafe_strerror(int code, char * ebp)
 
 
 /* Following macro from D.R. Butenhof's POSIX threads book:
-   ISBN 0-201-63392-2 . [Highly recommended book.] */
+ * ISBN 0-201-63392-2 . [Highly recommended book.] Changed __FILE__
+ * to __func__ */
 #define err_exit(code,text) do { \
     char strerr_buff[STRERR_BUFF_LEN]; \
     fprintf(stderr, "%s at \"%s\":%d: %s\n", \
-        text, __FILE__, __LINE__, tsafe_strerror(code, strerr_buff)); \
+        text, __func__, __LINE__, tsafe_strerror(code, strerr_buff)); \
     exit(1); \
     } while (0)
 
@@ -534,10 +534,14 @@ read_write_thread(void * v_clp)
     sz = clp->bpt * clp->bs;
     seek_skip =  clp->seek - clp->skip;
     memset(rep, 0, sizeof(Rq_elem));
-    psz = getpagesize();
+#if defined(HAVE_SYSCONF) && defined(_SC_PAGESIZE)
+    psz = sysconf(_SC_PAGESIZE); /* POSIX.1 (was getpagesize()) */
+#else
+    psz = 4096;     /* give up, pick likely figure */
+#endif
     if (NULL == (rep->alloc_bp = (unsigned char *)malloc(sz + psz)))
         err_exit(ENOMEM, "out of memory creating user buffers\n");
-    rep->buffp = (unsigned char *)(((unsigned long)rep->alloc_bp + psz - 1) &
+    rep->buffp = (unsigned char *)(((uintptr_t)rep->alloc_bp + psz - 1) &
                                    (~(psz - 1)));
     /* Follow clp members are constant during lifetime of thread */
     rep->bs = clp->bs;
@@ -649,13 +653,13 @@ normal_in_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
     char strerr_buff[STRERR_BUFF_LEN];
 
     /* enters holding in_mutex */
-    while (((res = read(clp->infd, rep->buffp,
-                        blocks * clp->bs)) < 0) && (EINTR == errno))
+    while (((res = read(clp->infd, rep->buffp, blocks * clp->bs)) < 0) &&
+           ((EINTR == errno) || (EAGAIN == errno)))
         ;
     if (res < 0) {
         if (clp->in_flags.coe) {
             memset(rep->buffp, 0, rep->num_blks * rep->bs);
-            fprintf(stderr, ">> substituted zeros for in blk=%"PRId64" for "
+            fprintf(stderr, ">> substituted zeros for in blk=%" PRId64 " for "
                     "%d bytes, %s\n", rep->blk,
                     rep->num_blks * rep->bs,
                     tsafe_strerror(errno, strerr_buff));
@@ -695,12 +699,12 @@ normal_out_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
     char strerr_buff[STRERR_BUFF_LEN];
 
     /* enters holding out_mutex */
-    while (((res = write(clp->outfd, rep->buffp,
-                 rep->num_blks * clp->bs)) < 0) && (EINTR == errno))
+    while (((res = write(clp->outfd, rep->buffp, rep->num_blks * clp->bs))
+            < 0) && ((EINTR == errno) || (EAGAIN == errno)))
         ;
     if (res < 0) {
         if (clp->out_flags.coe) {
-            fprintf(stderr, ">> ignored error for out blk=%"PRId64" for "
+            fprintf(stderr, ">> ignored error for out blk=%" PRId64 " for "
                     "%d bytes, %s\n", rep->blk,
                     rep->num_blks * rep->bs,
                     tsafe_strerror(errno, strerr_buff));
@@ -829,7 +833,7 @@ sg_in_operation(Rq_coll * clp, Rq_elem * rep)
         if (1 == res)
             err_exit(ENOMEM, "sg starting in command");
         else if (res < 0) {
-            fprintf(stderr, ME "inputting to sg failed, blk=%"PRId64"\n",
+            fprintf(stderr, ME "inputting to sg failed, blk=%" PRId64 "\n",
                     rep->blk);
             status = pthread_mutex_unlock(&clp->in_mutex);
             if (0 != status) err_exit(status, "unlock in_mutex");
@@ -900,7 +904,7 @@ sg_out_operation(Rq_coll * clp, Rq_elem * rep)
         if (1 == res)
             err_exit(ENOMEM, "sg starting out command");
         else if (res < 0) {
-            fprintf(stderr, ME "outputting from sg failed, blk=%"PRId64"\n",
+            fprintf(stderr, ME "outputting from sg failed, blk=%" PRId64 "\n",
                     rep->blk);
             status = pthread_mutex_unlock(&clp->out_mutex);
             if (0 != status) err_exit(status, "unlock out_mutex");
@@ -929,8 +933,8 @@ sg_out_operation(Rq_coll * clp, Rq_elem * rep)
                 guarded_stop_both(clp);
                 return;
             } else
-                fprintf(stderr, ">> ignored error for out blk=%"PRId64" for "
-                        "%d bytes\n", rep->blk, rep->num_blks * rep->bs);
+                fprintf(stderr, ">> ignored error for out blk=%" PRId64
+                        " for %d bytes\n", rep->blk, rep->num_blks * rep->bs);
             /* fall through */
         case 0:
             if (rep->dio_incomplete || rep->resid) {
@@ -969,8 +973,8 @@ sg_start_io(Rq_elem * rep)
 
     if (sg_build_scsi_cdb(rep->cmd, cdbsz, rep->num_blks, rep->blk,
                           rep->wr, fua, dpo)) {
-        fprintf(stderr, ME "bad cdb build, start_blk=%"PRId64", blocks=%d\n",
-                rep->blk, rep->num_blks);
+        fprintf(stderr, ME "bad cdb build, start_blk=%" PRId64
+                ", blocks=%d\n", rep->blk, rep->num_blks);
         return -1;
     }
     memset(hp, 0, sizeof(struct sg_io_hdr));
@@ -988,13 +992,14 @@ sg_start_io(Rq_elem * rep)
     if (dio)
         hp->flags |= SG_FLAG_DIRECT_IO;
     if (rep->debug > 8) {
-        fprintf(stderr, "sg_start_io: SCSI %s, blk=%"PRId64" num_blks=%d\n",
+        fprintf(stderr, "sg_start_io: SCSI %s, blk=%" PRId64 " num_blks=%d\n",
                rep->wr ? "WRITE" : "READ", rep->blk, rep->num_blks);
         sg_print_command(hp->cmdp);
     }
 
     while (((res = write(rep->wr ? rep->outfd : rep->infd, hp,
-                         sizeof(struct sg_io_hdr))) < 0) && (EINTR == errno))
+                         sizeof(struct sg_io_hdr))) < 0) &&
+           ((EINTR == errno) || (EAGAIN == errno)))
         ;
     if (res < 0) {
         if (ENOMEM == errno)
@@ -1025,7 +1030,8 @@ sg_finish_io(int wr, Rq_elem * rep, pthread_mutex_t * a_mutp)
     io_hdr.pack_id = (int)rep->blk;
 
     while (((res = read(wr ? rep->outfd : rep->infd, &io_hdr,
-                        sizeof(struct sg_io_hdr))) < 0) && (EINTR == errno))
+                        sizeof(struct sg_io_hdr))) < 0) &&
+           ((EINTR == errno) || (EAGAIN == errno)))
         ;
     if (res < 0) {
         perror("finishing io on sg device, error");
@@ -1054,7 +1060,7 @@ sg_finish_io(int wr, Rq_elem * rep, pthread_mutex_t * a_mutp)
             {
                 char ebuff[EBUFF_SZ];
 
-                snprintf(ebuff, EBUFF_SZ, "%s blk=%"PRId64,
+                snprintf(ebuff, EBUFF_SZ, "%s blk=%" PRId64,
                          wr ? "writing": "reading", rep->blk);
                 status = pthread_mutex_lock(a_mutp);
                 if (0 != status) err_exit(status, "lock aux_mutex");
